from responses import GET

from recipe_scrapers.gousto import Gousto
from tests import ScraperTest


class TestGoustoScraper(ScraperTest):

    scraper_class = Gousto

    @classmethod
    def expected_requests(cls):
        yield GET, "https://gousto.co.uk/cookbook/creamy-pork-tagliatelle", "tests/test_data/gousto.testhtml"

    def test_host(self):
        self.assertEqual("gousto.co.uk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Creamy Pork Tagliatelle")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Gousto")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "10g fresh parsley",
                "1/2 beef stock cube",
                "1 pot of double cream (227ml)",
                "200g linguine",
                "2 British pork loin steaks",
                "30g rennet-free parmesan",
                "1 garlic clove",
                "150g chestnut mushrooms",
                "1 onion",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 200°C/ 180°C (fan)/ 400°F/ Gas 6 Boil a kettle (used in steps 3 & 5)\nAdd 1-2 tbsp of olive oil to a pan on a high heat Once the oil is hot, sear the pork tenderloin for 2 min on each side, or until browned all over (keep the pan for step 6) Season to your taste with salt and pepper\nPlace the pork on an oven-proof tray and put in the oven for 8-10 min, or until cooked through Once done, remove from the oven, cover well and allow to rest until step 7 Dissolve half the (whole) beef stock cube in 100ml (200ml) of boiling water\nCut the mushrooms into quarters Peel the onion(s) and chop finely Crush the garlic with the side of a knife, peel it and chop finely Chop the parsley finely\nCook the pasta in a pot of boiling water and salt (optional) for 8-10 min or until the pasta is cooked to your taste, stirring occasionally, then drain\nReturn the pork pan to a medium-high heat, add the garlic and onion and cook for 1 min Add the mushrooms and cook for 1 min Reduce the heat, add the stock and 50ml (100ml) of cream and simmer for 5 min Add the parsley\nGrate the parmesan Cut the pork into slices (approx. 2cm) Season the mushroom sauce to your taste with salt and pepper\nAdd the pasta to the sauce and mix well Serve the pasta with the pork on top and sprinkle with parmesan Enjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class HomeChef(AbstractScraper):
    @classmethod
    def host(cls):
        return "homechef.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return "\n".join(
            [
                element.get("name") + "\n\n" + element.get("description")
                for element in self.schema.data.get("recipeInstructions").get(
                    "itemListElement"
                )
            ]
        )

    def description(self):
        return self.schema.description()



from recipe_scrapers.simplyquinoa import SimplyQuinoa
from tests import ScraperTest


class TestSimplyQuinoaScraper(ScraperTest):

    scraper_class = SimplyQuinoa

    def test_host(self):
        self.assertEqual("simplyquinoa.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.simplyquinoa.com/gluten-free-pancake-recipe/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "The Best Gluten-Free Pancakes")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Alyssa")

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("14 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup quinoa flour",
                "1 cup oat flour",
                "2 teaspoons baking powder",
                "1 1/4 cup almond milk ((or milk of choice))",
                "2 large eggs ((or flax eggs))",
                "2 tablespoons maple syrup",
                "2 tablespoons oil",
                "1/3 cup blueberries",
                "1/3 cup chocolate chips",
                "2 tablespoons lemon zest",
                "1 cup grated apple (from 1 small apple)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Whisk together the dry ingredients.\nIn a separate bowl, whisk together the wet ingredients.\nPour the wet ingredients into the dry and stir to combine. Once you have a smooth batter, it's time to cook!\nPreheat a pan over medium-low heat. Use a little cooking spray to grease it. Spoon 1/4 cup of batter onto the pan and gently spread it out with the back of a spoon. Cook the pancakes until bubbles begin to form, about 1 minute, then flip and cook another 1 - 2 minutes. Repeat until all the batter has been used.\nServe the pancakes immediately with your favorite toppings. You can also freeze these pancakes and reheat them in the toaster oven.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from dataclasses import dataclass
from typing import List, Optional

from ._abstract import AbstractScraper
from ._exceptions import ElementNotFoundInHtml
from ._utils import get_minutes, get_yields, normalize_string


@dataclass
class IngredientGroup:
    ingredients: List[str]
    purpose: Optional[
        str
    ] = None  # this group of ingredients is {purpose} (e.g. "For the dressing")


class NIHHealthyEating(AbstractScraper):
    @classmethod
    def host(cls):
        return "healthyeating.nhlbi.nih.gov"

    def title(self):
        # This content must be present for all recipes on this website.
        return normalize_string(self.soup.h1.get_text())

    def total_time(self):
        # This content must be present for all recipes on this website.
        time_table = self.soup.find("table", {"class": "recipe_time_table"})

        if time_table is None:
            raise ElementNotFoundInHtml("Table with times was not found.")

        return sum([get_minutes(td) for td in time_table.find_all("td")])

    def yields(self):
        # This content must be present for all recipes on this website.
        time_table = self.soup.find("table", {"class": "recipe_time_table"})

        if time_table is None:
            raise ElementNotFoundInHtml(
                "Table with the number of servings that the recipe yields was not found."
            )

        i = 0
        for t in time_table.findAll("th"):
            if "Yields" in t:
                break
            i += 1

        if i >= len(time_table.findAll("td")):
            raise ElementNotFoundInHtml(
                "Table cells with servings that the recipe yields were not found."
            )

        return get_yields(time_table.find_all("td")[i])

    def image(self):
        # Optional content recipes on this website.
        img = self.soup.find("img", {"class": "recipe_image", "src": True})

        if img is None:
            raise ElementNotFoundInHtml("Image not found.")

        image_relative_url = img.get("src")

        if image_relative_url is None:
            raise ElementNotFoundInHtml("Image not found.")

        image_relative_url = f"https://{self.host()}{image_relative_url}"

        return image_relative_url

    def ingredient_groups(self) -> List[IngredientGroup]:
        # This content must be present for recipes on this website.
        ingredients_div = self.soup.find("div", {"id": "ingredients"})
        section = []

        if ingredients_div is None:
            raise ElementNotFoundInHtml("Ingredients not found.")

        # Find more than one lists of ingredients
        ingredients_h4_sections = ingredients_div.find_all("h4")

        # Ingredients are broken down into sections
        # https://healthyeating.nhlbi.nih.gov/recipedetail.aspx?linkId=11&cId=1&rId=5

        if len(ingredients_h4_sections) >= 2:
            ingredients_sections = ingredients_div.find_all("tr")
            for ingredients_section in ingredients_sections:
                items = ingredients_section.find("p").get_text().strip().split("\n")
                # create ingredient group for each section
                res = IngredientGroup(
                    ingredients=items,
                    purpose=normalize_string(ingredients_section.find("h4").get_text()),
                )
                section.append(res)
            return section

        # Default case
        ingredients_p = ingredients_div.findAll("p")
        ingredients = [normalize_string(para.get_text()) for para in ingredients_p]
        ingredients_list = [
            ing for ing in ingredients if not ing.lower().startswith("recipe cards")
        ]

        # Edge case: ingredents are a mix for single main ingredients and a single sub section
        # https://healthyeating.nhlbi.nih.gov/recipedetail.aspx?linkId=0&cId=10&rId=163

        if len(ingredients_h4_sections) == 1:
            items = (
                ingredients_div.find("h4")
                .find_next_sibling("p")
                .get_text()
                .strip()
                .split("\n")
            )
            group = IngredientGroup(
                purpose=normalize_string(ingredients_h4_sections[0].get_text()),
                ingredients=items,
            )
            section.append(group)
            section.append(IngredientGroup(ingredients=ingredients_list[:-1]))
            return section

        return [IngredientGroup(ingredients_list)]

    def ingredients(self) -> List[str]:
        results = []
        for ingredient_group in self.ingredient_groups():
            results.extend(ingredient_group.ingredients)
        return results

    def instructions(self):
        # This content must be present for recipes on this website.
        directions_div = self.soup.find("div", {"id": "recipe_directions"})

        if directions_div is None:
            raise ElementNotFoundInHtml("Instructions not found.")

        instructions = directions_div.findAll("div", {"class": "steptext"})

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def nutrients(self):
        elements = []
        nutrition = {}

        for s in (
            self.soup.find("div", {"id": "nutrition_info"}).find("table").find_all("tr")
        ):
            for element in s.find_all("td"):
                if element.get_text().strip() != "":
                    elements.append(normalize_string(element.get_text()))

        for i in range(0, len(elements), 2):
            if len(elements) > i + 1:
                k, v = elements[i], elements[i + 1]
                nutrition[k] = v

        return nutrition

    def description(self):
        return normalize_string(
            self.soup.find("p", {"class": "recipe_detail_subtext"}).get_text()
        )

    def prep_time(self):
        return get_minutes(
            self.soup.find("table", {"class": "recipe_time_table"})
            .find_all("td")[0]
            .get_text()
        )

    def cook_time(self):
        return get_minutes(
            self.soup.find("table", {"class": "recipe_time_table"})
            .find_all("td")[1]
            .get_text()
        )

    def serving_size(self):
        return normalize_string(
            self.soup.find("table", {"class": "recipe_time_table"})
            .find_all("td")[3]
            .get_text()
        )

    def recipe_source(self):
        return normalize_string(
            self.soup.find("div", {"id": "Recipe_Source"}).get_text().split(": ")[1]
        )

    def recipe_cards(self):
        recipe_cards_maker = self.soup.find("strong", string="Recipe Cards:")

        if recipe_cards_maker is None:
            return None

        recipe_cards = []
        recipe_cards_maker_siblings = recipe_cards_maker.next_siblings
        for recipe_cards_maker_sibling in recipe_cards_maker_siblings:
            link = recipe_cards_maker_sibling.find("a")
            if recipe_cards_maker_sibling.name == "li":
                recipe_cards.append(
                    {
                        "size": normalize_string(recipe_cards_maker_sibling.get_text()),
                        "url": link.get("href"),
                    }
                )
        return recipe_cards



from recipe_scrapers.rezeptwelt import Rezeptwelt
from tests import ScraperTest


class TestRezeptweltScraper(ScraperTest):

    scraper_class = Rezeptwelt

    def test_host(self):
        self.assertEqual("rezeptwelt.de", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Thermomix Rezeptentwicklung", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Nudelsalat", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("sonstige Hauptgerichte", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://de.rc-cdn.community.thermomix.com/recipeimage/mbc1phgz-e2d56-160794-cfcd2-x6yic6yp/69b775cf-693a-443e-b61e-990e025b3ef5/main/nudelsalat.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "100 g Käse",
                "2 Möhren, in Stücken",
                "100 g Weißkohl, in Stücken",
                "1 Paprika, rot, geviertelt",
                "500 g Nudeln, gekocht, z. B.Spiral",
                "50 g Gemüsebrühe",
                "Salz",
                "Pfeffer",
                "300 g Knoblauch, aioli",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        expected = "Käse und Gemüse in den Mixtopf geben, 5 Sek./Stufe 5 zerkleinern (auf das laufende Messer fallen lassen, Stufe 5) und in eine Schüssel umfüllen.\nGekochte Nudeln, Gemüsebrühe und als Salatsauce Knoblauchaioli unterheben, durchziehen lassen und vor dem Servieren abschmecken.\n"
        self.assertEqual(expected, self.harvester_class.instructions())

    def test_ratings(self):
        self.assertEqual(4.04, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Deutsch", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Nudelsalat, ein Rezept der Kategorie sonstige Hauptgerichte.",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("de_DE", self.harvester_class.language())



import unittest

from recipe_scrapers._utils import get_minutes


class TestUtils(unittest.TestCase):
    def test_get_minutes_english_description(self):
        text = "1 hour 15 mins"
        self.assertEqual(75, get_minutes(text))

    def test_get_minutes_english_description_with_and(self):
        text = "1h and 15mins"
        self.assertEqual(75, get_minutes(text))

    def test_get_minutes_english_abbreviation(self):
        text = "3h10m"
        self.assertEqual(190, get_minutes(text))

    def test_get_minutes_short_iso_format(self):
        text = "PT2H30M"
        self.assertEqual(150, get_minutes(text))

    def test_get_minutes_long_iso_format(self):
        text = "P0DT1H10M"
        self.assertEqual(70, get_minutes(text))

    def test_get_minutes_int_in_string_literal(self):
        text = "90"
        self.assertEqual(90, get_minutes(text))

    def test_get_minutes_fraction_in_hours_with_dot_notation(self):
        text = "1.5 hours"
        self.assertEqual(90, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_unicode_character_halves(self):
        text = "1½ hours"
        self.assertEqual(90, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_unicode_character_three_fours(self):
        text = "1¾ hours"
        self.assertEqual(105, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_unicode_character_one_fours(self):
        text = "1¼ hours"
        self.assertEqual(75, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_unicode_character_two_thirds(self):
        text = "1⅔ hours"
        self.assertEqual(100, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_digits_with_slash(self):
        text = "1 1/2 hours"
        self.assertEqual(90, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_digits_with_slash_three_fours(self):
        text = "1 3/4 hours"
        self.assertEqual(105, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_digits_with_slash_one_fours(self):
        text = "1 1/4 hours"
        self.assertEqual(75, get_minutes(text))

    def test_get_minutes_fraction_with_fraction_digits_with_slash_two_thirds(self):
        text = "1 2/3 hours"
        self.assertEqual(100, get_minutes(text))

    def test_get_minutes_handles_dashes(self):
        text = "15 - 20 minutes"
        self.assertEqual(20, get_minutes(text))

    def test_get_minutes_handles_to(self):
        text = "15 to 20 minutes"
        self.assertEqual(20, get_minutes(text))

    iso8601_fixtures = {
        "PT1H": 60,
        "PT20M": 20,
        "PT2H10M": 130,
        "PT0H9M30S": 10,
    }

    def test_get_minutes_handles_iso8601(self):
        for text, expected_minutes in self.iso8601_fixtures.items():
            with self.subTest(text=text):
                self.assertEqual(expected_minutes, get_minutes(text))

    def test_list_public_methods(self):
        from recipe_scrapers import AbstractScraper

        public_method_names = [
            method
            for method in dir(AbstractScraper)
            if callable(getattr(AbstractScraper, method))
            if not method.startswith("_") and method not in ["soup", "links", "to_json"]
        ]
        self.assertEqual(
            [
                "author",
                "canonical_url",
                "category",
                "cook_time",
                "cuisine",
                "description",
                "host",
                "image",
                "ingredients",
                "instructions",
                "instructions_list",
                "language",
                "nutrients",
                "prep_time",
                "ratings",
                "reviews",
                "site_name",
                "title",
                "total_time",
                "yields",
            ],
            public_method_names,
        )



# mypy: allow-untyped-defs

from recipe_scrapers.rosannapansino import RosannaPansino
from tests import ScraperTest


class TestRosannaPansinoScraper(ScraperTest):

    scraper_class = RosannaPansino

    def test_host(self):
        self.assertEqual("rosannapansino.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Marshmallow Fondant", self.harvester_class.title())

    def test_image(self):
        self.assertEqual(
            "http://cdn.shopify.com/s/files/1/0163/5948/9636/articles/1Y6A6281_grande.jpg?v=1567109086",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 bag (10 ounces) mini marshmallows",
                "3 tablespoons water",
                "6 cups powdered sugar",
                "Oil, for greasing your hands",
                "Food coloring gel (optional)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Pour the marshmallows into a microwave-safe bowl. Stir in the water to evenly coat the marshmallows.\nMicrowave for 30 seconds, then stir. Repeat this process two more times, or until the mixture is smooth.\nSift 3 cups of the powdered sugar into a large bowl and make a hole in the center.\nPour the melted marshmallow mixture onto the powdered sugar.\nSift the rest of the powdered sugar on top of the marshmallows.\nOil your hands to prevent the marshmallow from sticking to you. Knead in the sugar until you have the consistency of soft taffy and the fondant no longer sticks to your hands. If you are tinting the fondant, add food coloring now and knead until the color is fully incorporated. (Coat your hands with shortening so the colors don't stain them.)\nIf you are not using the fondant right away, store it tightly wrapped or in a plastic bag at room temperature.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "A delicious and easy Marshmallow Fondant recipe!\nMakes 24 ounces",
            self.harvester_class.description(),
        )



from recipe_scrapers.tasty import Tasty
from tests import ScraperTest


class TestTastyScraper(ScraperTest):

    scraper_class = Tasty

    def test_host(self):
        self.assertEqual("tasty.co", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://tasty.co/recipe/red-wine-braised-short-ribs-with-cashew-cauliflower-mash",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Red Wine-Braised Short Ribs With Cashew Cauliflower Mash Recipe by Tasty",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Crystal Hatch")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.buzzfeed.com/thumbnailer-prod-us-east-1/video-api/assets/236025.jpg?resize=1200:*",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 lb bone in beef short ribs, cut into 2-inch pieces",
                "1 tablespoon kosher salt",
                "2 teaspoons kosher salt",
                "1 ½ teaspoons freshly ground black pepper",
                "2 tablespoons avocado oil, divided",
                "1 large white onion, cut into 1 inch (2.5 cm) pieces",
                "4 celery stalks, cut into 1/2 inch (1 1/4 cm) diagonal pieces",
                "2 tablespoons tomato paste",
                "2 tablespoons all purpose flour",
                "2 cups dry red wine, such as cabernet sauvignon",
                "1 head garlic, halved crosswise",
                "6 small rainbow carrots, peeled and ends trimmed",
                "10 sprigs fresh thyme",
                "2 cups low sodium beef broth",
                "2 lb cauliflower, cut into florets",
                "1 cup raw cashews",
                "1 teaspoon garlic",
                "¼ cup fresh parsley, chopped, for garnish",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Liberally season the short ribs on all sides with 1 tablespoon of salt and the pepper.\nHeat 2 tablespoons of avocado oil in a 5-quart (5 liter) ceramic pot over medium-high heat until shimmering. Working in batches, add 6-7 pieces of short rib at a time to the pot and sear without disturbing for 2-3 minutes, until a golden brown crust forms. Turn the pieces and continue cooking until seared well on all sides. Remove the short ribs from the pan and set aside, adding another tablespoon of avocado oil to the pot if needed, and repeat with the remaining short ribs.\nReduce the heat to medium and add the onion, celery, and the remaining ½ teaspoon of salt. Sauté for 3-4 minutes, until the vegetables begin to soften and release moisture.\nAdd the tomato paste and stir to coat the vegetables, then cook for 2 minutes. Add the flour and stir to coat the vegetables, then cook for 2-3 minutes more, until deep red in color. Pour in the wine and stir to release any browned bits from the bottom of the pot.\nNestle the short ribs on top of the vegetables in the pot. Bring to a boil, then reduce the heat to medium-low and simmer until the liquid is reduced by about half, 15-20 minutes.\nPreheat the oven to 350˚F (180°C).\nNestle the garlic between the short ribs, then place the carrots and thyme sprigs on top. Pour in the beef stock.\nCover the pot and transfer to the oven. Bake for 2½ hours, until the meat is falling off the bone and fork tender.\nWhile the short ribs are in the oven, make the cauliflower mash: Fill a medium pot with about an inch of water and set a steamer basket inside. Cover and bring the water to a simmer over medium-high heat. Add the cauliflower to the basket and steam for 12-15 minutes, until mashable with the back of a fork, but not overcooked.\nWorking in batches, transfer the steamed cauliflower to a fine-mesh strainer set over a bowl. Using another bowl that is slightly smaller than the strainer, press the bowl into the cauliflower to press excess liquid from the cauliflower. Repeat with remaining cauliflower. Discard the liquid and set the cauliflower aside.\nAdd the cashews to the bowl of a food processor. Blend on high speed for 2-3 minutes, until the cashews are completely broken down and can be mashed into a paste between your fingers, scraping down the sides of the bowl as necessary.\nAdd the strained cauliflower, garlic powder, and salt. Blend until smooth and well combined with the cashew paste.\nTransfer to a serving bowl and serve warm.\nServe the braised short ribs and carrots with the cauliflower mash and garnish with parsley.\nEnjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(
            4.7,
            self.harvester_class.ratings(),
        )



from recipe_scrapers.pingodoce import PingoDoce
from tests import ScraperTest


class TestPingoDoceScraper(ScraperTest):

    scraper_class = PingoDoce

    def test_host(self):
        self.assertEqual("pingodoce.pt", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Arroz de tamboril", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 q.b. hortelã",
                "1 kg tamboril",
                "100 ml azeite",
                "2 unid. cebola grande",
                "5 dente alho",
                "1 q.b. coentros",
                "1 unid. pimento",
                "4 unid. tomate maduro",
                "1 unid. malagueta pequena",
                "3 c. de sopa polpa de tomate",
                "300 g arroz carolino",
                "2 c. de chá sal",
                "1 q.b. pimenta branca",
                "400 g miolo de camarão",
                "40 g manteiga",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Coza o tamboril já cortado em cubos. Reserve a água de cozedura.\nNum tacho largo refogue em azeite as cebolas, os alhos e os coentros picados.\nQuando a cebola estiver translúcida, junte ao refogado o pimento em tiras, o tomate em cubos e a malagueta picada, envolvendo bem os ingredientes.\nDilua a polpa de tomate na água de cozedura do tamboril e vá acrescentando, aos poucos, ao refogado.\nDepois de lavado, junte o arroz e tempere de sal e pimenta.\nA meio da cozedura do arroz, adicione o tamboril, previamente cozido, e o miolo de camarão.\nPor fim, envolva a manteiga, polvilhe com a hortelã picada.\nSirva o seu arroz de tamboril com fatias de pão torrado.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(2.0, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            "Malandrinho, como se quer, saboroso e fresco, graças aos coentros e à hortelã, este arroz de tamboril vai deliciá-lo. Siga a receita e impressione lá em casa.",
            self.harvester_class.description(),
        )



# mypy: allow-untyped-defs

import re

from ._abstract import AbstractScraper


class Chefnini(AbstractScraper):
    @classmethod
    def host(cls):
        return "chefnini.com"

    def author(self):
        return "chefNini"

    def title(self):
        return self.soup.find("span", {"itemprop": "headline"}).get_text()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return None

    def yields(self):
        recipe_yields_text = self.soup.find(
            "h3", {"itemprop": "recipeYield"}
        ).get_text()
        servings = re.match(r"Pour (\d+)", recipe_yields_text).group(1)
        return f"{servings} servings"

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = self.soup.findAll("li", {"itemprop": "ingredients"})
        return [ingredient.get_text() for ingredient in ingredients]

    def instructions(self):
        instruction_list = [
            instruction.get_text()
            for instruction in self.soup.find(
                "div", {"itemprop": "recipeInstructions"}
            ).children
            if instruction.get_text().strip()
        ]

        return "\n".join(instruction_list)

    def ratings(self):
        return None

    def cuisine(self):
        return None

    def description(self):
        return self.soup.find("p", {"itemprop": "description"}).get_text()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class SimplyCookit(AbstractScraper):
    @classmethod
    def host(cls):
        return "simply-cookit.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = []
        for li in self.soup.find("ul", {"class": "recipe_ingredients"}).findAll("li"):
            ingredients.append(normalize_string(li.get_text()))

        return ingredients

    def instructions(self):
        instructions = []
        for li in self.soup.find("ul", {"class": "recipe_steps"}).findAll("li"):
            li.find("span", {"class": "number"}).clear()
            instructions.append(normalize_string(li.get_text()))

        return "\n".join(instructions)

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.tastykitchen import TastyKitchen
from tests import ScraperTest


class TestTastyKitchenScraper(ScraperTest):

    scraper_class = TastyKitchen

    def test_host(self):
        self.assertEqual("tastykitchen.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.thepioneerwoman.com/food-cooking/recipes/a11059/restaurant-style-salsa/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Restaurant Style Salsa")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Ree | The Pioneer Woman")

    def test_total_time(self):
        self.assertEqual(10, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://tastykitchen.com/recipes/wp-content/uploads/sites/2/2010/01/TPW_5376.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 can (28 Ounce) Whole Tomatoes With Juice",
                "2 cans (10 Ounce) Rotel (diced Tomatoes And Green Chilies)",
                "¼ cups Chopped Onion",
                "1 clove Garlic, Minced",
                "1 whole Jalapeno, Quartered And Sliced Thin",
                "¼ teaspoons Sugar",
                "¼ teaspoons Salt",
                "¼ teaspoons Ground Cumin",
                "½ cups Cilantro (more To Taste!)",
                "½ whole Lime Juice",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Note: this is a very large batch. Recommend using a 12-cup food processor, or you can process the ingredients in batches and then mix everything together in a large mixing bowl.",
                    "Combine whole tomatoes, Rotel, onion, jalapeno, garlic, sugar, salt, cumin, lime juice, and cilantro in a blender or food processor. Pulse until you get the salsa to the consistency you’d like—I do about 10 to 15 pulses. Test seasonings with a tortilla chip and adjust as needed.",
                    "Refrigerate salsa for at least an hour. Serve with tortilla chips or cheese nachos.",
                ]
            ),
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.smulweb import Smulweb
from tests import ScraperTest


class TestSmulwebScraper(ScraperTest):

    scraper_class = Smulweb

    def test_host(self):
        self.assertEqual("smulweb.nl", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("pwiggers", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Lekker! Ovenschotel spitskool champignons spekreepjes en prei",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual("Ovenschotel", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://d1mlbwr23caxox.cloudfront.net/public/sites/default/files/styles/recipe_teaser/public/recipe-images/IMG_1442.JPG_16.jpg?VersionId=HiJx912VlnCnUKiAYHKDqDsZLjsSy9zI",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertCountEqual(
            [
                "Plm. 750 gram spitskool",
                "1 prei",
                "250 gram champignons",
                "150 gram mager gerookte spekreepjes of baconreepjes",
                "1 eetlepel neutrale olie",
                "peper, zout",
                "1/2 theelepel gemalen nootmuskaat",
                "cayennepeper naar smaak",
                "1 + 1 eetlepel mosterd",
                "200 ml crème fraîche",
                "750 gram kruimige aardappelen",
                "50 geraspte belegen kaas",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertCountEqual(
            [
                "Verwarm de oven voor op 200 graden.",
                "Verhit de olie in de wok.",
                "Bak hierin de spekreepjes knapperig.",
                "Voeg de plakjes champignons toe en bak deze mee tot al het vocht is verdwenen.",
                "Voeg de prei toe en bak deze plm. 2 minuten mee.",
                "Doe dan de spitskool erbij en roerbak alles nog plm. 5 - 10 minuten, afhankelijk van de dikte van de spitskool. (De spitskool moet nog knapperig blijven!)",
                "Maak de spitskool op smaak met peper, zout, nootmuskaat en wat cayennepeper.",
                "Roer 2 eetlepels crème fraîche en 1 eetlepel mosterd door de kool.",
                "Doe de kool in een vuurvaste ovenschool.",
                "Laat de gare aardappels goed droog stomen.",
                "Stamp de aardappels met een stamper grof en roer de rest van de crème fraîche en 1 eetlepel mosterd door de aardappelen.",
                "Strijk de aardappelen uit over de spitkool en bestrooi met wat geraspte kaas.",
                "Zet de schaal in de voorverwarmde oven tot de kaas een goudbruine kleur heeft gekregen.",
                "Als de ingrediënten nog warm zijn kun je de schaal ook even onder de grill zetten tot de kaas goudbruin is.",
            ],
            self.harvester_class.instructions_list(),
        )

    def test_ratings(self):
        self.assertEqual(3.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Nederlands", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Lekker pittige ovenschotel. Door het gebruik van de mosterd en crème fraîche krijgt de kool een lekkere romige smaak. Geen aardappelpuree bovenop, maar aardappelen gestampt met wat crème fraîche en mosterd erdoor geroerd. Hierdoor krijg je een wat grovere laag, die heerlijk is bij de spitskool.Ook lekker met plakjes ontbijtspek op de schotel. Ze worden lekker knapperig in de oven. (zie foto)",
            self.harvester_class.description(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.weightwatchers import WeightWatchers
from tests import ScraperTest


class TestWeightwatchersScraper(ScraperTest):

    # Test-Url:
    # https://cmx.weightwatchers.de/details/WWRECIPE:562a9bc8a43e6bde2cf369df

    scraper_class = WeightWatchers
    test_file_name = "weightwatchers_2"

    def test_host(self):
        self.assertEqual("www.weightwatchers.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("WeightWatchers", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Hackbraten Stroganoff")

    def test_category(self):
        self.assertEqual("WeightWatchers", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(65, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(25, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(40, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cmx.weightwatchers.com/assets-proxy/weight-watchers/image/upload/t_WINE_EXTRALARGE/vziguy9ale25vtx7spgl.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "125 g Champignons, frisch",
                "5 Stück Cornichons",
                "1 Bund Frühlingszwiebeln/Lauchzwiebeln",
                "600 g Tatar, roh",
                "2 EL Paniermehl/Semmelbrösel",
                "1 Stück, Gewichtsklasse M Eier, Hühnereier",
                "4 TL Senf, klassisch",
                "1 Prise(n) Salz/Jodsalz",
                "1 Prise(n) Pfeffer",
                "600 g Kartoffeln; festkochend",
                "2 TL Halbfettmargarine, 39 % Fett",
                "1 EL Weizenmehl",
                "300 ml Gemüsebouillon/Gemüsebrühe, zubereitet; (2 TL Instantpulver)",
                "3 EL Crème légère",
                "2 EL, gehackt Petersilie",
                "430 g Rote Bete (Konserve); entspricht 1 Glas",
            ],
            self.harvester_class.ingredients(),
        )

    def test_ingredients_count(self):
        self.assertEqual(16, len(self.harvester_class.ingredients()))

    def test_instructions(self):
        self.assertEqual(
            "Backofen auf 200° C (Gas: Stufe 3, Umluft: 180° C) vorheizen. Champignons trocken abreiben und mit Cornichons in feine Würfel schneiden. Frühlingszwiebeln waschen und in Ringe schneiden. Champignon-, Cornichonwürfel und Frühlingszwiebelringe mit Tatar, Paniermehl, Ei, 1 Teelöffel Senf, Salz und Pfeffer verkneten und zu einem Braten formen.\nHackbraten in eine Kastenform (Länge 30 cm) geben und im Backofen auf unterer Schiene ca. 45 Minuten garen. Kartoffeln schälen, halbieren und in Salzwasser ca. 20 Minuten garen. Für die Sauce Margarine in einem Topf schmelzen, Mehl darin hellgelb anschwitzen, unter Rühren mit Brühe ablöschen, ca. 4 Minuten köcheln lassen. Mit Créme légère verfeinern und mit Salz, Pfeffer, restlichem Senf und Petersilie würzen. Kartoffeln abgießen und Rote Bete abtropfen lassen. Hackbraten mit Senfsauce, Salzkartoffeln und Rote Bete servieren.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "Wenn das mal keine mutige Abwandlung des beliebten Klassiker ist. Schmeckt übrigens ganz vorzüglich!",
            self.harvester_class.description(),
        )

    def test_difficulty(self):
        self.assertEqual("Mittel", self.harvester_class.difficulty())

    def test_nutrients(self):
        expected_nutrients = {"personal points": "10 personal points"}
        self.assertEqual(self.harvester_class.nutrients(), expected_nutrients)



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class LAtelierDeRoxane(AbstractScraper):
    @classmethod
    def host(cls):
        return "latelierderoxane.com"

    def image(self):
        return self.soup.find("meta", {"property": "og:image", "content": True}).get(
            "content"
        )

    def title(self):
        div = self.soup.find("div", {"class": "bloc_titreh1 bloc_blog"})
        return div.find("h1").get_text()

    def description(self):
        div = self.soup.find("div", {"class": "bloc_chapeau bloc_blog"})
        return div.find("p").get_text()

    def total_time(self):
        return get_minutes(self.get_bloc_temps_value_by_index(0))

    def prep_time(self):
        return get_minutes(self.get_bloc_temps_value_by_index(1))

    def cook_time(self):
        return get_minutes(self.get_bloc_temps_value_by_index(2))

    def yields(self):
        return get_yields(self.get_bloc_temps_value_by_index(4))

    def ingredients(self):
        raw_ingredients = self.soup.find_all("div", {"class": "ingredient"})
        formatted_ingredients = []
        for ingredient in raw_ingredients:
            formatted_ingredients.append(normalize_string(ingredient.get_text()))
        return formatted_ingredients

    def instructions(self):
        instruction_bloc = self.soup.find(
            "div", {"class": "bloc_texte_simple bloc_blog"}
        )
        instructions = instruction_bloc.find_all("li")
        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def site_name(self):
        return "L'Atelier de Roxane"

    def get_bloc_temps_value_by_index(self, i):
        div_bloc = self.soup.find("div", {"class": "bloc_temps bloc_blog"})
        div_infos = div_bloc.find("div", {"class": "infos"})
        span_value = div_infos.find_all("span", {"class": "valeur"})
        return span_value[i].get_text()



from recipe_scrapers.hassanchef import HassanChef
from tests import ScraperTest


class TestHassanChef(ScraperTest):

    scraper_class = HassanChef

    def test_host(self):
        self.assertEqual("hassanchef.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.hassanchef.com/2019/06/chicken-lollipop-recipe-lollipop-chicken.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Chicken lollipop recipe")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Mobasir Hassan")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "For chicken lollipop dry",
                "6-8 pieces of chicken wings.",
                "1/2 teaspoon red chilly sauce.",
                "1/2 teaspoon schezwan sauce.",
                "1/2 teaspoon red chilly paste",
                "1/2 teaspoon chopped garlic",
                "1/2 teaspoon chopped ginger",
                "1 teaspoon chopped celery",
                "1/2 teaspoon green chilly paste",
                "1/2 cup corn flour",
                "1/4 cup Maida",
                "1 egg beaten",
                "1/2 teaspoon salt",
                "1/2 teaspoon white pepper powder",
                "1/2 teaspoon lemon juice",
                "For masala chicken lollipop",
                "1/2 teaspoon red chilly paste",
                "1/3 teaspoon schezwan sauce",
                "1/2 teaspoon chopped garlic",
                "1/2 teaspoon ginger chopped",
                "1/3 cup chopped onions.",
                "Small handful chopped spring onions.",
                "1 teaspoon cooking oil",
                "Salt as per taste",
                "2 teaspoon corn flour slurry",
                "For chicken lollipop gravy",
                "1 cup water",
                "1/3 teaspoon dark soya sauce",
                "1/4 teaspoon red chilly sauce",
                "1/3 teaspoon red chilly paste",
                "1/4 teaspoon tomato soup",
                "1/4 teaspoon schezwan sauce",
                "1/3 teaspoon green chilli paste",
                "1/3 teaspoon white pepper powder",
                "small pinch of sugar",
                "3 teaspoon corn flour slurry",
                "salt as per taste",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "How to make the recipe\nMake lollipop batter\nTake corn flour and all purpose flour in a mixing bowl and mix the chopped ginger, chopped garlic, red chilly paste, red chilly sauce, schezwan sauce, green chilli paste and chopped celery.\nThen add salt, white pepper powder, lemon juice and beaten egg.\nGive everything a fine mix to make a smooth batter. If required add little amount of water.\nMake chicken lollipop dry\nWash and clean the lollipop pieces and pat them dry completely.\nMix these lollipops with the batter evenly so that all parts are covered with the batter\nLet them marinate for 2 to 4 hours or overnight.\nHeat oil in a deep vessel pan or kadai in medium flame.\nBring the marinated lollipops and remove excess marinate from them\nPour then into the oil one by one carefully. Keep the heat flame to low\nFry the lollipops for 7 to 8 minutes in slow flame till crisp golden.\nRemove them in an absorbent paper to absorb any excess oil from them.\nRemove them in an absorbent paper to absorb any excess oil from them.\nGarnish a plate and serve the chicken lollipop dry with schezwan sauce.\nMake masala chicken lollipop\nHeat half teaspoon oil in a wok or deep vessel pan in high flame.\nAdd half teaspoon of ginger and garlic and saute. Then add half of the chopped onions and fry them in high flame\nWhen onions become translucent lower the heat and add red chilly paste and schezwan sauce. Mix and pour little water. Stir everything well.\nNow add the fried chicken lollipops and mix well. Add 1/3 teaspoon white pepper powder and salt as per taste\nThen pour corn flour slurry into the above mixture and continue to stir.\nWhen the mixture become little dry and the masala coated the lollipops evenly switch off the flame and remove them in a serving plate. Garnish with chopped spring onions and serve\nMake chicken lollipop gravy\nHeat 1/2 teaspoon oil in a wok in high flame and add the chopped ginger, garlic and onion. Fry them till onions become translucent in high flame.\nLower the flame and add 1/3 teaspoon red chilly paste, 1/4 teaspoon red chilly sauce, 1/4 teaspoon tomato ketchup and schezwan sauce. Mix everything\nAdd 1 cup water into the above mixture and simmer for a while.\nThen add salt as per taste, 1/4 teaspoon white pepper powder, 1/3 teaspoon green chili paste and a small pinch of sugar.\nFurther add 1/3 teaspoon dark soya sauce into the mixture.\nNow pour the corn flour slurry, mix and summer for one minute. The gravy will start to become thick and glossy.\nAdd the fried chicken lollipops and further simmer for a minute. Sprinkle chipped spring onions into the gravy\nCheck the seasoning and thickness of the gravy and remove to serve with fried rice and noodles.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class EatingWell(AbstractScraper):
    @classmethod
    def host(cls):
        return "eatingwell.com"

    def title(self):
        return self.schema.title()

    def author(self):
        return self.soup.find("span", {"class": "author-name"}).get_text()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def total_time(self):
        div = self.soup.findAll("div", {"class": "recipe-meta-item"})
        d = {
            normalize_string(key): normalize_string(value)
            for key, value in [i.text.split(":") for i in div]
            if value is not None
        }
        return get_minutes(d.get("total"))

    def yields(self):
        div = self.soup.findAll("div", {"class": "recipe-meta-item"})
        d = {
            normalize_string(key): normalize_string(value)
            for key, value in (i.text.split(":") for i in div)
            if value is not None
        }
        return get_yields(d.get("Servings"))



from recipe_scrapers.purelypope import PurelyPope
from tests import ScraperTest


class TestPurelyPopeScraper(ScraperTest):

    scraper_class = PurelyPope

    def test_host(self):
        self.assertEqual("purelypope.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://purelypope.com/sweet-chili-brussel-sprouts/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Sweet Chili Brussel Sprouts")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://purelypope.com/wp-content/uploads/2020/05/IMG_5412-1-150x150.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups brussel sprouts, stems removed & cut in half",
                "2 tbsp coconut aminos",
                "1 tbsp sriracha",
                "1/2 tbsp maple syrup",
                "1 tsp sesame oil",
                "Everything bagel seasoning or sesame seeds, to top",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Instructions\n\nBrussel Sprout Time!\n\nPreheat oven to 350 degrees.\nWhisk the sauce (coconut aminos, sriracha, maple syrup & sesame oil) together in a large bowl.\nToss in brussel sprouts and coat mixture evenly over the brussels.\nRoast for 30 minutes.\nTurn oven to broil for 2-3 minutes to crisp (watch carefully to not burn.)\nTop with everything or sesame seeds.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.tasteofhome import TasteOfHome
from tests import ScraperTest


class TestTasteOfHomeScraper(ScraperTest):

    scraper_class = TasteOfHome

    def test_host(self):
        self.assertEqual("tasteofhome.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Taste of Home", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.tasteofhome.com/recipes/pressure-cooker-sauerbraten/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Pressure-Cooker Sauerbraten")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://tmbidigitalassetsazure.blob.core.windows.net/rms3-prod/attachments/37/1200x1200/Pressure-Cooker-Sauerbraten_EXPS_THN18_39243_E06_06_2b.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 whole cloves",
                "4 whole peppercorns",
                "1 bay leaf",
                "1/2 cup water",
                "1/2 cup white vinegar",
                "2 teaspoons sugar",
                "1/2 teaspoon salt",
                "Dash ground ginger",
                "1 pound boneless beef top round steak, cut into 1-inch cubes",
                "3 medium carrots, cut into 1/2-inch slices",
                "2 celery ribs, cut into 1/2-inch slices",
                "1 small onion, chopped",
                "1/3 cup crushed gingersnaps",
                "Hot cooked egg noodles",
                "Optional: Chopped fresh parsley and coarsely ground pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Place cloves, peppercorns and bay leaf on a double thickness of cheesecloth; bring up corners of cloth and tie with kitchen string to form a bag. In a large bowl, combine the water, vinegar, sugar, salt and ginger. Add beef and spice bag; let stand at room temperature for 30 minutes.\nTransfer all to a 6-qt. electric pressure cooker. Add carrots, celery and onion. Lock the lid and close pressure-release valve. Adjust to pressure-cook on high for 10 minutes. Quick-release pressure. Select saute setting and adjust for medium heat; bring liquid to a boil. Discard the spice bag. Stir in gingersnaps; cook and stir until thickened, about 3 minutes. Serve with egg noodles. If desired, top with parsley and pepper.\nFreeze option: Freeze cooled sauerbraten in freezer containers. To use, partially thaw in refrigerator overnight. Heat through in a saucepan, stirring occasionally; add a little broth or water if necessary.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.25, self.harvester_class.ratings())



from recipe_scrapers.globo import Globo
from tests import ScraperTest


class TestGloboScraper(ScraperTest):

    scraper_class = Globo

    def test_host(self):
        self.assertEqual("receitas.globo.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://receitas.globo.com/strogonoff-de-frango-simples-4fbe8cc656ec5b3c9801b7e5.ghtml",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Strogonoff de frango", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Alm3")

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://s2.glbimg.com/hXxPtwuPHqQUp2w3pdklGQHBznM=/696x390/smart/filters:cover():strip_icc()/i.s3.glbimg.com/v1/AUTH_1f540e0b94d8437dbbc39d567a1dee68/internal_photos/bs/2022/8/O/xH9h1GSnW1LhMobyL7hQ/strogonoff-de-frango-receita.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(50, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 colheres de sopa de óleo",
                "1 tablete de caldo de galinha",
                "1 quilo de peito de frango em cubos",
                "2 colheres de sopa de molho de tomate",
                "2 colheres de sopa de mostarda",
                "2 colheres de sopa de ketchup",
                "Champignon a gosto",
                "1 lata de creme de leite sem soro",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "1 Em uma panela, coloque 3 colheres de sopa de óleo e 1 tablete de caldo de galinha. Espere aquecer para dissolver o tablete.\n2 Em seguida, adicione 1 quilo de peito de frango em cubos e deixe dourar.\n3 Depois, acrescente 2 colheres de sopa de molho de tomate, 2 colheres de sopa de mostarda, 2 colheres de sopa de ketchup e champignon a gosto. Misture.\n4 Desligue o fogo e acrescente 1 lata de creme de leite. Misture novamente.\n5 Sirva em seguida.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.forksoverknives import ForksOverKnives
from tests import ScraperTest


class TestForksOverKnives(ScraperTest):

    scraper_class = ForksOverKnives

    def test_host(self):
        self.assertEqual("forksoverknives.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Darshana Thacker Wendel", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Butternut Squash Mac and Cheese with Broccoli",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual("Pasta & Noodles", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 cups", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.forksoverknives.com/wp-content/uploads/butternut-broccoli-mac-and-cheese-wordpress.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 medium butternut squash (1¾ lb.)",
                "1 onion, finely chopped (1 cup)",
                "4 cloves garlic, minced",
                "½ teaspoon finely chopped fresh thyme",
                "2 cups unsweetened, unflavored plant milk, such as almond, soy, cashew, or rice",
                "2 tablespoons nutritional yeast",
                "1 tablespoon white wine vinegar",
                "¼ teaspoon sea salt",
                "⅛ teaspoon freshly ground black pepper",
                "3 cups dried whole grain penne pasta (8 oz.)",
                "3 cups small broccoli florets",
                "Fresh basil leaves",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Peel squash; halve squash and remove seeds. Cut squash into large pieces. Place squash pieces in a steamer basket in a large pan. Add water to saucepan to just below basket. Bring to boiling. Steam, covered, about 12 minutes or until tender.\nHeat a large saucepan over medium. Add onion, garlic, thyme, and ¼ cup water to pan. Cook about 10 minutes or until onion is tender, stirring occasionally and adding water, 1 to 2 Tbsp. at a time, as needed to prevent sticking.\nTransfer onion mixture to a blender. Add squash and the next five ingredients (through pepper). Cover and blend until smooth. Pour squash mixture into a large saucepan.\nCook pasta according to package directions, adding broccoli the last 5 minutes of cooking; drain. Add drained pasta and broccoli to squash mixture; toss to coat. Serve warm topped with fresh basil.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.13, self.harvester_class.ratings())

    def test_language(self):
        self.assertEqual("en-US", self.harvester_class.language())



from recipe_scrapers.comidinhasdochef import ComidinhasDoChef
from tests import ScraperTest


class TestComidinhasDoChefScraper(ScraperTest):
    scraper_class = ComidinhasDoChef

    def test_host(self):
        self.assertEqual("comidinhasdochef.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Pedro Cavalcanti", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "https://comidinhasdochef.com/coxa-de-frango-na-fritadeira-eletrica/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "Coxa de Frango na Fritadeira Elétrica", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("5 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://comidinhasdochef.com/wp-content/uploads/2022/01/Coxa-de-Frango-na-Fritadeira-Elétrica00.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "500 g de coxas de frango",
                "1 colher (sopa) de mostarda",
                "1 colher (sopa) de azeite",
                "2 colheres (sopa) de shoyo",
                "1 unidade de limões espremidos",
                "1 colher (sopa) de salsinha desidratada",
                "1 dente de alho",
                "0 e 1/2 colher (sopa) de colorau",
                "pimenta do reino a gosto",
                "sal a gosto",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Como preparar Coxa de Frango na Fritadeira Elétrica\nPasso 1\nEm uma tigela coloque as coxas de frango e adicione todos os ingredientes ( a mostarda, o azeite, o shoyu, o suco de limão, a salsa, o alho picado, o colorau, a pimenta do reino e o sal);\nPasso 2\nCom as mãos misture bem, massageando as coxas de frango para que o tempero pegue bem;\nPasso 3\nEm seguida tampe a tigela e reserve por 25-30 minutos;\nPasso 4\nPré aqueça sua fritadeira elétrica a 200º C por 5 minutos;\nPasso 5\nColoque as coxas de frango e deixe fritar por 10 minutos;\nPasso 6\nAbra a fritadeira, vire o frango e deixe por mais 10 minutos, mantendo sempre a temperatura de 200º C;\nPasso 7\nRetire as coxas de frango da fritadeira e sirva em seguida.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.9, self.harvester_class.ratings())



from recipe_scrapers.panelinha import Panelinha
from tests import ScraperTest


class TestPanelinhaScraper(ScraperTest):
    scraper_class = Panelinha
    test_file_name = "panelinha_2"

    def test_host(self):
        self.assertEqual("panelinha.com.br", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Arroz sírio com frango", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Panelinha")

    def test_total_time(self):
        self.assertEqual(0, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.panelinha.com.br/receita/1433732400000-Arroz-sirio-com-frango.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 bifes de filé de peito de frango (cerca de 240 g)",
                "⅓ de xícara (chá) de arroz",
                "⅔ de xícara (chá) de lentilha",
                "1 cebola",
                "1 dente de alho",
                "2 xícaras (chá) de água",
                "1 ½ colher (sopa) de azeite",
                "½ colher (chá) de pimenta síria",
                "1 colher (chá) de sal",
                "1 pitada de açúcar",
                "¼ de xícara (chá) de nozes picadas",
                "⅓ de xícara (chá) de iogurte natural",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        expected_instructions = (
            "Coloque a lentilha numa tigela funda e cubra com 1 xícara (chá) de água fervente. Deixe de molho enquanto prepara os outros ingredientes.\n"
            "Descasque e fatie a cebola em meias-luas médias. Descasque e pique fino o alho. Corte os bifes de frango em tirinhas de cerca de 1 cm x 7 cm.\n"
            "Leve ao fogo médio uma panela média. Quando aquecer, junte 1 colher (sopa) de azeite e a cebola fatiada. Tempere com uma pitada de sal e de açúcar e abaixe o fogo. Deixe cozinhar por cerca de 10 minutos, mexendo de vez enquanto, até a cebola ficar bem dourada - não aumente o fogo para acelerar o processo, caso contrário, a cebola pode queimar em vez de caramelizar.\n"
            "Transfira a cebola para uma tigela e aumente o fogo para médio. Acrescente o restante do azeite e doure as tirinhas de frango aos poucos - se colocar todas ao mesmo tempo, elas vão soltar o próprio líquido e cozinhar no vapor, em vez de dourar. Tempere com uma pitada de sal e mexa aos poucos para dourar por igual.\n"
            "Junte a cebola dourada e o alho e misture por apenas 1 minuto. Acrescente o arroz, 1 colher (chá) de sal e a pimenta síria. Mexa bem para envolver os grãos nos temperos.\n"
            "Numa peneira, escorra a lentilha e junte à panela. Cubra com 2 xícaras (chá) de água, misture e deixe cozinhar. Assim que começar a ferver, diminua o fogo e deixe cozinhar com a tampa entreaberta até a água secar, por cerca de 20 minutos.\n"
            "Desligue o fogo e mantenha a panela tampada por 5 minutos antes de servir, para que os grãos terminem de cozinhar no próprio vapor. Divida o arroz em dois pratos e salpique com as nozes. Sirva a seguir com iogurte natural."
        )

        return self.assertEqual(
            expected_instructions,
            self.harvester_class.instructions(),
        )



from recipe_scrapers.mykitchen101en import MyKitchen101en
from tests import ScraperTest


class TestMyKitchen101enScraper(ScraperTest):

    scraper_class = MyKitchen101en

    def test_host(self):
        self.assertEqual("mykitchen101en.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Mykitchen101en Team", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Baked Mini Egg Sponge Cakes", self.harvester_class.title())

    def test_yields(self):
        self.assertEqual("30 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://mykitchen101en.com/wp-content/uploads/2020/11/mini-baked-egg-sponge-cake-mykitchen101en-feature.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 eggs (grade A/size: L)",
                "125 g fine sugar",
                "⅛ tsp fine salt",
                "115 g plain flour",
                "20 g cornstarch",
                "30 g melted butter",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Preheat oven to 200°C/395°F.\nBeat eggs & sugar\nBeat eggs lightly, add in salt and sugar, beat until combined.\nPrepare warm water\nCombine 1 Liter of plain water with 400 ml of hot water (from electric thermo pot) in a large steel bowl to yield warm water with temperature about 45°C/113°F.\nbeat egg mixture until stiff\nDip mixing bowl in warm water bath, beat egg mixture over medium-high speed until stiff (about 5 minutes). (Reminder: Beating egg mixture over warm water will shorten the time for it to become stiff.)\nSift flour\nCombine plain flour and cornstarch, sift twice.\nAdd flour to egg mixture, add in melted butter\nAdd flour to egg mixture gradually, mix over low speed, fold until well mixed with spatula. Add in melted butter gradually, fold gently until mixed with spatula.\nPour batter into piping bags.\nFill batter in mini muffin pans\nLine mini muffin pans (diameter = 4.8 cm, depth = 2.2 cm) with cupcake liners, fill with batter until 80% full.\nBake in the preheated oven at 190°C/375°F for 20-22 minutes, until golden brown. (Reminder: The heat for different oven is different, the suggested time is only for reference, adjust the baking time base on your oven if necessary. If the 1st batch has 24 pieces of cakes, the 2nd batch will only have 6 pieces, then the baking time for 2nd batch can be shortened, just bake until they are golden brown.)\nUnmould & cool\nUnmould and cool mini egg sponge cakes on wire rack.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.simplyrecipes import SimplyRecipes
from tests import ScraperTest


class TestSimplyRecipesScraper(ScraperTest):

    scraper_class = SimplyRecipes

    def test_host(self):
        self.assertEqual("simplyrecipes.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.simplyrecipes.com/recipes/one_pot_chicken_and_rice_soup/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "One-Pot Chicken and Rice Soup")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Aaron Hutcherson")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        # 6 to 8 servings (makes about 3 quarts), debatable it should be 8 servings.
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 bone-in chicken breasts, skin removed (1 to 1 1/2 pounds)",
                "1 cup long-grain white rice, like basmati",
                "2 ribs celery, diced small",
                "2 medium carrots, peeled and diced small",
                "1 onion, diced small",
                "2 cloves garlic, peeled but left whole",
                "1 teaspoon salt, plus more to taste (see Recipe Note)",
                "1/2 teaspoon ground black pepper",
                "2 quarts unsalted or low-sodium chicken stock",
                "Juice of 1/2 lemon (about 3 tablespoons)",
                "Chopped fresh parsley, for serving",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Assemble the soup and bring to a simmer: Combine the chicken, white rice, celery, carrots, onions, garlic, salt, and pepper in a large pot. Add the chicken stock and bring to a boil over high heat. Once the soup is boiling, reduce the heat to keep the soup at a gentle simmer.",
                    "Simmer for about 25 minutes: As the soup simmers, skim off any foam that collects on the surface with a spoon. Continue to simmer until the rice and vegetables are tender, about 25 minutes. Remove the soup from heat.",
                    "Shred the chicken and make the garlic paste: Remove the chicken and garlic cloves using a slotted spoon or tongs. Transfer the chicken breasts to a bowl and shred with two forks. Discard the bones. Return the shredded chicken to the pot. Smash the garlic cloves into a paste against a cutting board using a fork or the flat of your knife. Stir the paste back into the soup.",
                    "Season and serve: Stir the lemon juice into to the soup, and taste. The soup should taste rich, barely salty, and with just a hint of lemon. Add more salt or lemon juice as needed until it tastes good to you. Divide among bowls and sprinkle with some parsley for serving. Leftovers will keep for about a week refrigerated, or up to 3 months frozen.",
                ]
            ),
            self.harvester_class.instructions(),
        )



from recipe_scrapers.paleorunningmomma import PaleoRunningMomma
from tests import ScraperTest


class TestPaleoRunningMommaScraper(ScraperTest):

    scraper_class = PaleoRunningMomma

    def test_host(self):
        self.assertEqual("paleorunningmomma.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Michele Rosen", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Paleo Beef Stroganoff {Whole30, Keto}",  # noqa
            self.harvester_class.title(),
        )

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.paleorunningmomma.com/wp-content/uploads/2020/12/beef-stroganoff-9.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "5 Tbsp ghee (divided)",
                "1.5 lb top sirloin or tenderloin (thinly sliced into 1/2” x 2” strips)",
                "1 large onion (thinly sliced)",
                "Sea salt and black pepper to taste",
                "8 oz mushrooms (sliced)",
                "3 cloves garlic (minced)",
                "2 cups beef broth",
                "1 Tbsp coconut aminos",
                "2 Tbsp arrowroot flour (or tapioca)",
                "Salt to taste and black pepper to taste",
                "1 cup coconut cream (canned, unsweetened)",
                "1 Tbsp fresh lemon juice",
                "1 tsp dijon mustard",
                "Pinch Sea salt",
                "Sea salt and black pepper to taste",
                "Minced parsley (for garnish)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "In a large deep skillet, melt 2 tablespoons of the ghee over medium-high heat.\nAdd the steak in a single layer, season with salt and pepper, and let it cook for 3 minutes to brown. Flip, and cook on the other side until browned, another 2 minutes. Then remove steak from pan with a slotted spoon, transfer to a plate, and set aside.\nLower the heat to medium and the remaining 3 tablespoons of ghee to the skillet. Once it has melted, add the onions and sauté for about 3 minutes, until soft and fragrant. Add mushrooms and sauté for an additional 5-7 minutes, stirring occasionally, or until the mushrooms are cooked and the onions are soft, then add the garlic and sauté 1 minute, stirring occasionally.\nIn a bowl or large measuring cup, whisk in together the beef broth, coconut aminos and arrowroot until smooth. Pour the mixture into the skillet and stir well to combine. Simmer for about 5 minutes, stirring occasionally. Meanwhile, in a separate small bowl whisk together the coconut cream, lemon juice, mustard and a pinch of sea salt. Stir the coconut cream mixture plus the cooked steak into the skillet and stir until combined. Taste and season with additional salt and pepper if needed, and cook just long enough to heat through.\nGarnish with parsley, if desired, and serve over cauliflower rice or your favorite veggie noodles. Enjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.15, self.harvester_class.ratings())



# mypy: allow-untyped-defs

from ._utils import normalize_string
from .weightwatchers import WeightWatchers


# collect the differences between public and non-public weightwatcher recipes in this class
class WeightWatchersPublic(WeightWatchers):
    @classmethod
    def host(cls):
        return "www.weightwatchers.com"

    def _find_data_container(self):
        return self.soup.find("div", {"class": "HorizontalList_list__GESs0"})

    def _extract_item_field(self, item):
        return item.find("div", {"data-e2e-name": "attribute_item_value"})

    def image(self):
        return self.soup.find("img", {"class": "FoodMasthead_heroImage__BjVdZ"})["src"]

    def nutrients(self):
        return {
            "points": self.soup.find("div", {"class": "Coin_text__3UOb0"})["aria-label"]
        }

    def description(self):
        return normalize_string(
            self.soup.find("div", {"data-e2e-name": "food_masthead_detail_description"})
            .find("div", {"class": "ReadMoreLess_collapsed__IAzxP"})
            .get_text()
        )

    def instructions(self):
        return self._get_instructions(
            "h2", "class", "InstructionsFood_headline__vw7cn", "span"
        )

    def _find_ingredient_tags(self):
        return (
            self.soup.find("div", {"class": "IngredientsCard_card__VSY4x"})
            .find("div", {"data-e2e-name": "vertical_list_items"})
            .find_all("div", recursive=False)
        )

    @staticmethod
    def _extract_ingredient_name(ingredient):
        return normalize_string(
            ingredient.find("p", {"data-e2e-name": "ingredient_name"}).get_text()
        )

    @staticmethod
    def _extract_portion_parts(ingredient):
        tags = ingredient.find(
            "p", {"data-e2e-name": "ingredient_description"}
        ).find_all("span")

        if len(tags) > 2:
            comment = normalize_string(tags[2].get_text().replace(", ", "", 1))
            unit = normalize_string(tags[1].get_text())
        else:
            description_parts = normalize_string(tags[1].get_text()).split(", ", 1)
            unit = description_parts[0]
            comment = description_parts[1] if len(description_parts) > 1 else None

        return normalize_string(tags[0].get_text()), unit, comment



from recipe_scrapers.bodybuilding import Bodybuilding
from tests import ScraperTest


class TestBodybuildingScraper(ScraperTest):

    scraper_class = Bodybuilding

    def test_host(self):
        self.assertEqual("bodybuilding.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual(
            "Shannon Clark and Danielle Christine", self.harvester_class.author()
        )

    def test_title(self):
        self.assertEqual("Spicy Beans and Rice", self.harvester_class.title())

    def test_category(self):
        self.assertEqual(
            "Side Dishes,Vegetarian,High-Protein", self.harvester_class.category()
        )

    def test_total_time(self):
        self.assertEqual(75, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.bodybuilding.com/images/2021/october/spicy-beans-and-rice-header-960x540.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 tbsp olive oil",
                "2 clove garlic, minced",
                "1 red onion, diced",
                '1 stalk, medium (7-1/2" - 8" long) celery, peeled and chopped',
                "1 cup mushrooms, sliced",
                "1 15 oz. can red kidney beans",
                "1 tbsp hot sauce",
                "1 tsp paprika",
                "2½ cup low-sodium vegetable broth",
                "1 tsp vegetable bouillon",
                "1 cup wild rice, uncooked",
                "1 tbsp grass-fed butter",
                "1 tbsp green onion, chopped",
                "1 salt and pepper to taste",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """1. Dice the celery, red onion, and mushrooms.
2. Heat the butter in a skillet over medium heat. Add all of the vegetables and stir-fry for 3-4 minutes or until vegetables are tender. Set aside.
3. Bring broth to a boil in a large pot. Stir in wild rice, hot sauce, paprika, and bouillon and cook 45-60 minutes or until rice is tender.
4. Add olive oil and garlic to a skillet and quickly sauté. Remember that garlic burns quickly, so it is important to keep a close eye on it. Add in the cooked rice, beans, and cooked vegetables.
5. Stir-fry for 3-5 minutes to crisp up the rice and serve. Garnish with chopped green onion.
6. Optional: Add any other vegetables you prefer.""",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "This one-dish meal has it all—flavorful wild rice, protein-rich beans, heart-healthy vegetables, and just the perfect amount of spice to make you come back for more every time. This dish is also quick and easy to prepare, making it ideal for those who just don't have time to dedicate hours to preparing lunch or dinner.",
            self.harvester_class.description(),
        )



from recipe_scrapers.yummly import Yummly
from tests import ScraperTest


class TestYummlyScraper(ScraperTest):

    scraper_class = Yummly

    def test_host(self):
        self.assertEqual("yummly.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.yummly.com/recipe/Easy-White-Cheese-_-Garlic-Pizzas-9351327",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Easy White Cheese & Garlic Pizzas"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Sara Mellas")

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 pieces naan",
                "2 tablespoons extra-virgin olive oil",
                "3 cloves garlic",
                "1 cup shredded mozzarella cheese (or shredded pizza blend cheese, 1 cup is 4 oz.)",
                "4 ounces ricotta cheese (4 oz. is 1 scant cup)",
                "1 1/2 teaspoons Italian seasoning",
                "2 tablespoons shaved Parmesan cheese (for serving)",
                "basil leaves (for garnish, optional)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Preheat oven to 425°F.",
                    "Line a baking sheet with parchment paper.",
                    "Arrange the naan on the baking sheet. Drizzle an equal amount of olive oil over the surface of each piece.",
                ]
            ),
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class PopSugar(AbstractScraper):
    @classmethod
    def host(cls):
        return "popsugar.com"

    def title(self):
        title = self._context().find("h2", {"class": "recipe-title"}).get_text()
        return normalize_string(title)

    def total_time(self):
        anchor = self._context().find(string="Total Time")
        time = anchor.parent.findNext("dd").get_text()
        return get_minutes(time)

    def yields(self):
        anchor = self._context().find(string="Yield")
        serves = anchor.parent.findNext("dd").get_text()
        return get_yields(serves)

    def image(self):
        article = self.soup.find("article")
        if article:
            article = article.get("data-share-image")
        return article

    def ingredients(self):
        container = self._context().find("h3", string="Ingredients").parent
        entries = container.findAll("li")

        ingredients = []
        skip_next = False
        for entry in entries:
            for item in entry.contents:
                if skip_next:
                    skip_next = False
                    continue

                # Each item is an ingredient entry unless it is a tag. The tags are filtered from the ingredient
                # list. If the tag is a <b> tag the next item is also ignored in order to skip headings.
                if item.name is not None:
                    skip_next = item.name == "b"
                    continue

                ingredients.append(normalize_string(item))

        return ingredients

    def instructions(self):
        container = self._context().find("h3", string="Directions").parent
        return "\n".join([entry.get_text() for entry in container.findAll("li")])

    def _context(self):
        return self.soup.find("div", {"class": "recipe-card"})



from recipe_scrapers.spendwithpennies import SpendWithPennies
from tests import ScraperTest


class TestSpendWithPenniesScraper(ScraperTest):

    scraper_class = SpendWithPennies

    def test_host(self):
        self.assertEqual("spendwithpennies.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.spendwithpennies.com/gooey-chocolate-pudding-cake/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Gooey Chocolate Pudding Cake")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Holly Nilsson")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.spendwithpennies.com/wp-content/uploads/2013/10/Chocolate-Pudding-Cake-SpendWithPennies-B-4.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "¾ cup all-purpose flour",
                "½ cup sugar",
                "½ cup unsweetened cocoa powder",
                "1 ½ teaspoons baking powder",
                "⅔ cup milk",
                "2 tablespoons vegetable oil",
                "1 teaspoon vanilla",
                "⅔ cup brown sugar (packed)",
                "¼ cup cocoa powder",
                "¼ cup miniature semisweet chocolate chips",
                "1 ¼ cups very hot water",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 350°F.\nIn a 2 qt casserole dish, combine flour, white sugar, cocoa powder, and baking powder.\nAdd milk and oil, and vanilla. Stir until well mixed.\nIn a small bowl, combine brown sugar, cocoa powder and chocolate chips. Sprinkle over cake batter. DO NOT STIR. Pour hot water over top.\nBake for 30-35 minutes or until the top looks cooked. Serve warm (with ice cream if desired).",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.9, self.harvester_class.ratings())



from recipe_scrapers.monsieurcuisine import MonsieurCuisine
from tests import ScraperTest


class TestMonsieurCuisineScraper(ScraperTest):

    scraper_class = MonsieurCuisine

    def test_host(self):
        self.assertEqual("monsieur-cuisine.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("© Monsieur Cuisine", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Guacamole", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(5, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.monsieur-cuisine.com/fileadmin/_processed_/e/8/csm_23055_Rezeptfoto_01_2fae43be2a.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 brins de coriandre",
                "100 g de tomates",
                "1 petit oignon",
                "2 avocats bien mûrs",
                "Jus de ½ citron vert",
                "1 c.c. de harissa (pâte de piment rouge)",
                "1 pincée de sel",
                "1 pincée de poivre",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Rincer et sécher la coriandre. Dans le bol mixeur, hacher finement les tiges entières avec la touche Turbo/7 secondes. Rincer les tomates, les couper en deux en ôtant le pédoncule et les épépiner. Les ajouter dans le bol mixeur et les hacher grossièrement 8 secondes/vitesse 5. Réserver dans un saladier.\nPeler l’oignon, le couper en deux, puis le hacher finement touche Turbo/7 secondes. Peler l’avocat et détacher la chair du noyau. L’ajouter avec le jus de citron vert, la harissa, le sel et le poivre. Puis mixer 8 secondes/vitesse 6. Ajouter enfin les tomates et mixer le tout avec le programme Sens inverse/10 secondes/vitesse 2.\nDresser le guacamole dans un saladier et servir.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())



from recipe_scrapers.countryliving import CountryLiving
from tests import ScraperTest


class TestCountryLivingScraper(ScraperTest):

    scraper_class = CountryLiving

    def test_host(self):
        self.assertEqual("countryliving.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.countryliving.com/food-drinks/a32042602/roasted-mushroom-and-bacon-dutch-baby/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Roasted Mushroom and Bacon Dutch Baby"
        )

    def test_total_time(self):
        self.assertEqual(70, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 lb. mixed mushrooms (such as cremini, beech, or shiitake), roughly chopped 4 slices bacon, sliced",
                "3 large eggs",
                "1 clove garlic, chopped",
                "3/4 c. whole milk",
                "3 tbsp. unsalted butter, melted, divided",
                "1/2 c. all-purpose flour, spooned and leveled",
                "2 tbsp. cornstarch",
                "Kosher salt",
                "1 tbsp. fresh thyme",
                "2 oz. fontina or Cheddar cheese, grated (about 1/2 cup)",
                "1 scallion, thinly sliced",
                "2 tbsp. chopped flat-leaf parsley",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Set oven racks in middle and upper positions. Preheat oven to 450°F. Place mushrooms and bacon on a rimmed baking sheet. Place on top rack in oven while preheating, and roast, stirring once, until mushrooms are golden brown, 25 to 30 minutes. Place a 10-inch cast-iron skillet on middle rack and heat 15 minutes.\nPlace eggs and garlic in a blender. Process on high until frothy, 45 seconds. With blender running, gradually add milk and 2 tablespoons butter; stop blender. Add flour, cornstarch, and 1/4 teaspoon salt; process 1 minute. Fold in thyme.\nCarefully add remaining tablespoon butter to heated skillet and swirl to coat. Immediately add batter. Bake until golden brown and puffed, 14 to 16 minutes. Sprinkle with cheese and bake until melted, 3 to 5 minutes.\nTop with mushroom mixture, scallions, and parsley.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.domesticateme import DomesticateMe
from tests import ScraperTest


class TestDomesticateMeScraper(ScraperTest):

    scraper_class = DomesticateMe

    def test_host(self):
        self.assertEqual("domesticate-me.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://domesticate-me.com/dude-diet-buffalo-chicken-quinoa-bake/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "The Dude Diet: Buffalo Chicken Quinoa Bake"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Serena Wolf")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://domesticate-me.com/wp-content/uploads/2018/03/Buffalo-Chicken-Quinoa-Bake-2.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_rating(self):
        self.assertEqual(4.5, self.harvester_class.ratings())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cup quinoa",
                "1½ cups low-sodium chicken broth",
                "1 tablespoon extra-virgin olive oil",
                "1½ cups cauliflower “rice” (aka very finely chopped cauliflower florets)",
                "½ medium yellow onion (minced)",
                "½ cup finely chopped carrots",
                "½ cup finely chopped celery",
                "2 cups diced or shredded chicken breast",
                "½ cup Frank’s Red Hot Buffalo Wing Sauce (plus extra for serving)",
                "¾ cup grated sharp cheddar cheese",
                "¾ cup grated provolone cheese (Gouda is also great!)",
                "¼ cup whole-wheat panko breadcrumbs",
                "3 scallions (thinly sliced (optional))",
                "For the Yogurt Ranch:",
                "1½ cups nonfat plain Greek yogurt",
                "1 teaspoon dried parsley (crushed (Just use your fingers to crush the flakes.))",
                "½ teaspoon dried dill weed",
                "½ teaspoon kosher salt",
                "¼ teaspoon garlic powder",
                "¼ teaspoon black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Combine the quinoa and chicken broth in a small saucepan and bring to a boil. Lower to a simmer, cover the saucepan with a lid, and cook for 14 minutes, or until all of the liquid has been absorbed. Let the quinoa rest, covered, for 5 minutes, then fluff with a fork.\nMeanwhile, whip up the yogurt ranch! In a medium bowl, whisk all the ingredients for the ranch. Briefly set that deliciousness aside.\nPre-heat the oven to 375 degrees.\nHeat the olive oil in a large ovenproof skillet or shallow Dutch oven over medium heat. When the oil is hot and shimmering, add the cauliflower, onion, carrot, and celery. Cook for 5 minutes until the onion is translucent and the vegetables are tender. Add the cooked quinoa, chicken, and Frank’s to the pan and fold everything together. Turn off the heat and fold in 1 cup of the yogurt ranch and half of the cheddar and provolone. Taste the filling. Add a little extra Frank’s if you deem it necessary.\nSmooth the top of the filling with a spatula. Add the remaining cheese in an even layer and sprinkle with the panko.\nBake for 25 minutes until the cheese is melted and bubbling. If you want to brown the top of the bake (I DO!!), pop the casserole under the broiler for 1 to 2 minutes until the bread crumbs turn golden brown.\nWhisk a tablespoon or so of water into the remaining yogurt ranch just until it has a drizzle-able consistency. Serve the quinoa bake drizzled with as much extra ranch and Frank’s as you like and garnish with scallions.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.thehappyfoodie import TheHappyFoodie
from tests import ScraperTest


class TestTheHappyFoodie(ScraperTest):

    scraper_class = TheHappyFoodie
    test_file_name = "thehappyfoodie_1"

    def test_host(self):
        self.assertEqual("thehappyfoodie.co.uk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Poulet rôti au vin rouge (Roast Red Wine Chicken)",
        )

    def test_total_time(self):
        self.assertEqual(90, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "150ml red wine",
                "100g tomato paste",
                "3 sprigs of thyme, leaves picked",
                "3 sprigs of marjoram, leaves picked, or ½ teaspoon dried",
                "100ml red wine vinegar",
                "1 whole chicken cut into 8 pieces (approx. 1.5kg)",
                "Salt and ground black pepper",
                "500g baby potatoes, washed",
                "3 onions, peeled and cut into quarters",
                "6 carrots, peeled and cut into quarters lengthways",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Mix together the red wine, tomato paste, herbs and red wine vinegar. Season the chicken pieces with plenty of salt and pepper then place in a bag with the marinade. Shake the bag to make sure each piece is well coated. Place in the fridge for at least 30 minutes.\nIn the meantime, place the potatoes in a pan of cold water, put the lid on top and bring up to the boil. Boil for 1–2 minutes, then drain in a colander. Place the onions, carrots and potatoes in a large baking dish or tray (big enough to fit the chicken and the vegetables) and pour over 125ml of water. Preheat the oven to 200°C. Remove the chicken from the fridge and arrange the pieces, skin side up, in a layer on top of the vegetables in the dish. Pour the rest of the marinade over the chicken. Cover with a sheet of baking paper or foil and roast in the preheated oven for 30 minutes. Remove the baking paper or foil and baste the chicken with the cooking liquid. Roast, uncovered, for another 15 minutes or until the skin is crisp. Serve immediately.\nLes petits conseils – tips: Buying a whole chicken always works out more affordable. If you aren’t up for dissecting it yourself, ask your butcher to cut it into pieces for you. Otherwise if there’s no knife-wielding butcher about you can always cheat and go for chicken thighs. If you’re unsure whether the chicken is cooked through, pierce with a sharp knife and the juices from the chicken should come out clear.\nFaire en avance – get ahead: The veg and chicken can be prepared up to a day in advance, then simply pop it all in the baking tray and cook as indicated in the recipe.\nMarinating time: 30 minutes – overnight\n",
            self.harvester_class.instructions(),
        )

    def test_author(self):
        return self.assertEqual("Rachel Khoo", self.harvester_class.author())

    def test_image(self):
        return self.assertEqual(
            "https://thehappyfoodie.co.uk/wp-content/uploads/2021/08/9780718177478_roast_red_wine_chicken_s900x0_c3840x2243_l0x1385.jpg",
            self.harvester_class.image(),
        )

    def test_cuisine(self):
        return self.assertEqual("French", self.harvester_class.cuisine())

    def test_category(self):
        return self.assertEqual("Dinner, Main Course", self.harvester_class.category())



# mypy: disallow_untyped_defs=False
# IF things in this file continue get messy (I'd say 300+ lines) it may be time to
# find a package that parses https://schema.org/Recipe properly (or create one ourselves).


from itertools import chain

import extruct

from recipe_scrapers.settings import settings

from ._exceptions import SchemaOrgException
from ._utils import get_minutes, get_yields, normalize_string

SCHEMA_ORG_HOST = "schema.org"

SYNTAXES = ["json-ld", "microdata"]


class SchemaOrg:
    @staticmethod
    def _contains_schematype(item, schematype):
        itemtype = item.get("@type", "")
        itemtypes = itemtype if isinstance(itemtype, list) else [itemtype]
        return schematype.lower() in "\n".join(itemtypes).lower()

    def __init__(self, page_data, raw=False):
        if raw:
            self.format = "raw"
            self.data = page_data
            return
        self.format = None
        self.data = {}

        data = extruct.extract(
            page_data,
            syntaxes=SYNTAXES,
            errors="log" if settings.LOG_LEVEL <= 10 else "ignore",
            uniform=True,
        )

        for syntax in SYNTAXES:
            # make sure entries of type Recipe are always parsed first
            syntax_data = data.get(syntax, [])
            try:
                index = [x.get("@type", "") for x in syntax_data].index("Recipe")
                syntax_data.insert(0, syntax_data.pop(index))
            except ValueError:
                pass

            for item in syntax_data:
                if SCHEMA_ORG_HOST not in item.get("@context", ""):
                    continue

                # If the item itself is a recipe, then use it directly as our datasource
                if self._contains_schematype(item, "Recipe"):
                    self.format = syntax
                    self.data = item
                    return

                # Check for recipe items within the item's entity graph
                for graph_item in item.get("@graph", []):
                    if self._contains_schematype(graph_item, "Recipe"):
                        self.format = syntax
                        self.data = graph_item
                        return

                # If the item is a webpage and describes a recipe entity, use the entity as our datasource
                if self._contains_schematype(item, "WebPage"):
                    main_entity = item.get("mainEntity", {})
                    if self._contains_schematype(main_entity, "Recipe"):
                        self.format = syntax
                        self.data = main_entity
                        return

    def language(self):
        return self.data.get("inLanguage") or self.data.get("language")

    def title(self):
        return normalize_string(self.data.get("name"))

    def category(self):
        category = self.data.get("recipeCategory")
        if isinstance(category, list):
            return ",".join(category)
        return category

    def author(self):
        author = self.data.get("author") or self.data.get("Author")
        if (
            author
            and isinstance(author, list)
            and len(author) >= 1
            and isinstance(author[0], dict)
        ):
            author = author[0]
        if author and isinstance(author, dict):
            author = author.get("name")
        if author:
            return author.strip()

    def total_time(self):
        if not (self.data.keys() & {"totalTime", "prepTime", "cookTime"}):
            raise SchemaOrgException("Cooking time information not found in SchemaOrg")

        def get_key_and_minutes(k):
            source = self.data.get(k)
            # Workaround: strictly speaking schema.org does not provide for minValue (and maxValue) properties on objects of type Duration; they are however present on objects with type QuantitativeValue
            # Refs:
            #  - https://schema.org/Duration
            #  - https://schema.org/QuantitativeValue
            if type(source) == dict and "minValue" in source:
                source = source["minValue"]
            return get_minutes(source, return_zero_on_not_found=True)

        total_time = get_key_and_minutes("totalTime")
        if not total_time:
            times = list(map(get_key_and_minutes, ["prepTime", "cookTime"]))
            total_time = sum(times)
        return total_time

    def cook_time(self):
        if not (self.data.keys() & {"cookTime"}):
            raise SchemaOrgException("Cooktime information not found in SchemaOrg")
        return get_minutes(self.data.get("cookTime"), return_zero_on_not_found=True)

    def prep_time(self):
        if not (self.data.keys() & {"prepTime"}):
            raise SchemaOrgException("Preptime information not found in SchemaOrg")
        return get_minutes(self.data.get("prepTime"), return_zero_on_not_found=True)

    def yields(self):
        if not (self.data.keys() & {"recipeYield", "yield"}):
            raise SchemaOrgException("Servings information not found in SchemaOrg")
        yield_data = self.data.get("recipeYield") or self.data.get("yield")
        if yield_data and isinstance(yield_data, list):
            yield_data = yield_data[0]
        recipe_yield = str(yield_data)
        return get_yields(recipe_yield)

    def image(self):
        image = self.data.get("image")

        if image is None:
            raise SchemaOrgException("Image not found in SchemaOrg")

        if isinstance(image, list):
            # Could contain a dict
            image = image[0]

        if isinstance(image, dict):
            image = image.get("url")

        if "http://" not in image and "https://" not in image:
            # some sites give image path relative to the domain
            # in cases like this handle image url with class methods or og link
            image = ""

        return image

    def ingredients(self):
        ingredients = (
            self.data.get("recipeIngredient") or self.data.get("ingredients") or []
        )

        if ingredients and isinstance(ingredients[0], list):
            ingredients = list(chain(*ingredients))  # flatten

        return [
            normalize_string(ingredient) for ingredient in ingredients if ingredient
        ]

    def nutrients(self):
        nutrients = self.data.get("nutrition", {})

        # Some recipes contain null or numbers which breaks normalize_string()
        # We'll ignore null and convert numbers to a string, like Schema validator does
        for key, val in nutrients.copy().items():
            if val is None:
                del nutrients[key]
            elif type(val) in [int, float]:
                nutrients[key] = str(val)

        return {
            normalize_string(nutrient): normalize_string(value)
            for nutrient, value in nutrients.items()
            if nutrient != "@type" and value is not None
        }

    def _extract_howto_instructions_text(self, schema_item):
        instructions_gist = []
        if type(schema_item) is str:
            instructions_gist.append(schema_item)
        elif schema_item.get("@type") == "HowToStep":
            if schema_item.get("name", False):
                # some sites have duplicated name and text properties (1:1)
                # others have name same as text but truncated to X chars.
                # ignore name in these cases and add the name value only if it's different from the text
                if not schema_item.get("text").startswith(
                    schema_item.get("name").rstrip(".")
                ):
                    instructions_gist.append(schema_item.get("name"))
            instructions_gist.append(schema_item.get("text"))
        elif schema_item.get("@type") == "HowToSection":
            name = schema_item.get("name") or schema_item.get("Name")
            if name is not None:
                instructions_gist.append(name)
            for item in schema_item.get("itemListElement"):
                instructions_gist += self._extract_howto_instructions_text(item)
        return instructions_gist

    def instructions(self):
        instructions = self.data.get("recipeInstructions") or ""

        if instructions and isinstance(instructions[0], list):
            instructions = list(chain(*instructions))  # flatten

        if isinstance(instructions, list):
            instructions_gist = []
            for schema_instruction_item in instructions:
                instructions_gist += self._extract_howto_instructions_text(
                    schema_instruction_item
                )

            return "\n".join(
                normalize_string(instruction) for instruction in instructions_gist
            )

        return instructions

    def ratings(self):
        ratings = self.data.get("aggregateRating")
        if ratings is None:
            raise SchemaOrgException("No ratings data in SchemaOrg.")

        if isinstance(ratings, dict):
            ratings = ratings.get("ratingValue")

        if ratings is None:
            raise SchemaOrgException("No ratingValue in SchemaOrg.")

        return round(float(ratings), 2)

    def cuisine(self):
        cuisine = self.data.get("recipeCuisine")
        if cuisine is None:
            raise SchemaOrgException("No cuisine data in SchemaOrg.")
        elif isinstance(cuisine, list):
            return ",".join(cuisine)
        return cuisine

    def description(self):
        description = self.data.get("description")
        if description is None:
            raise SchemaOrgException("No description data in SchemaOrg.")
        if description and isinstance(description, list):
            description = description[0]
        return normalize_string(description)



from recipe_scrapers.cookpad import CookPad
from tests import ScraperTest


class TestCookPadScraper(ScraperTest):

    scraper_class = CookPad

    def test_host(self):
        self.assertEqual("cookpad.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://cookpad.com/recipe/4610651", self.harvester_class.canonical_url()
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "30分で簡単♡本格バターチキンカレー♡")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "reoririna")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.cpcdn.com/recipes/4610651/640x640c/6de3ac788480ce2787e5e39714ef0856?u=6992401&p=1519025894",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "♥鶏モモ肉 500g前後",
                "♥玉ねぎ 2個",
                "♥にんにくチューブ 5cm",
                "♥生姜チューブ 5cm(なくても♡)",
                "♥カレー粉 大さじ1と1/2",
                "♥バター 大さじ2+大さじ3(60g)",
                "＊トマト缶 1缶",
                "＊コンソメ 小さじ1",
                "＊塩 小さじ(1〜)2弱",
                "＊砂糖 小さじ2",
                "＊水 100ml",
                "＊ケチャップ 大さじ1",
                "♥生クリーム 100ml",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "鶏モモ肉 は一口大に、 玉ねぎ は薄切り(orみじん切り)にします♪\nフライパンに バター(大さじ2) を熱し、鶏肉 に 塩胡椒 をふり表面をこんがり焼きます♪\nお鍋に バター(大さじ3) にんにくチューブ 生姜チューブ 玉ねぎ を入れてあめ色になるまでじっくり炒めます♪\nカレー粉 を加えて弱火で3分くらい炒めます♪\n＊ と 鶏肉(油分も) を加えて沸騰したら火が通るまで(10分程)煮ます♪\n仕上げに 生クリーム を加えて混ぜ、温まったらすぐ火を止めます♪ 完成♡♡ 更に仕上げに生クリームをトッピングしました♡\n子供ごはんはこんな感じの盛り付けに♡♥",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
import inspect
from collections import OrderedDict
from typing import Dict, List, Optional, Tuple, Union
from urllib.parse import urljoin

import requests
from bs4 import BeautifulSoup

from recipe_scrapers.settings import settings

from ._schemaorg import SchemaOrg

# some sites close their content for 'bots', so user-agent must be supplied
HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:86.0) Gecko/20100101 Firefox/86.0"
}


class AbstractScraper:
    page_data: Union[str, bytes]

    def __init__(
        self,
        url: Union[str, None],
        proxies: Optional[
            Dict[str, str]
        ] = None,  # allows us to specify optional proxy server
        timeout: Optional[
            Union[float, Tuple[float, float], Tuple[float, None]]
        ] = None,  # allows us to specify optional timeout for request
        wild_mode: Optional[bool] = False,
        html: Union[str, bytes, None] = None,
    ):
        if html:
            self.page_data = html
            self.url = url
        else:
            assert url is not None, "url required for fetching recipe data"
            resp = requests.get(
                url,
                headers=HEADERS,
                proxies=proxies,
                timeout=timeout,
            )
            self.page_data = resp.content
            self.url = resp.url

        self.wild_mode = wild_mode
        self.soup = BeautifulSoup(self.page_data, "html.parser")
        self.schema = SchemaOrg(self.page_data)

        # attach the plugins as instructed in settings.PLUGINS
        if not hasattr(self.__class__, "plugins_initialized"):
            for name, func in inspect.getmembers(self, inspect.ismethod):
                current_method = getattr(self.__class__, name)
                for plugin in reversed(settings.PLUGINS):
                    if plugin.should_run(self.host(), name):
                        current_method = plugin.run(current_method)
                setattr(self.__class__, name, current_method)
            setattr(self.__class__, "plugins_initialized", True)

    @classmethod
    def host(cls) -> str:
        """get the host of the url, so we can use the correct scraper"""
        raise NotImplementedError("This should be implemented.")

    def canonical_url(self):
        canonical_link = self.soup.find("link", {"rel": "canonical", "href": True})
        if canonical_link:
            return urljoin(self.url, canonical_link["href"])
        return self.url

    def title(self):
        raise NotImplementedError("This should be implemented.")

    def category(self):
        raise NotImplementedError("This should be implemented.")

    def total_time(self):
        """total time it takes to preparate and cook the recipe in minutes"""
        raise NotImplementedError("This should be implemented.")

    def cook_time(self):
        """cook time of the recipe in minutes"""
        raise NotImplementedError("This should be implemented.")

    def prep_time(self):
        """preparation time of the recipe in minutes"""
        raise NotImplementedError("This should be implemented.")

    def yields(self):
        """The number of servings or items in the recipe"""
        raise NotImplementedError("This should be implemented.")

    def image(self):
        raise NotImplementedError("This should be implemented.")

    def nutrients(self):
        raise NotImplementedError("This should be implemented.")

    def language(self):
        """
        Human language the recipe is written in.

        May be overridden by individual scrapers.
        """
        candidate_languages = OrderedDict()
        html = self.soup.find("html", {"lang": True})
        candidate_languages[html.get("lang")] = True

        # Deprecated: check for a meta http-equiv header
        # See: https://www.w3.org/International/questions/qa-http-and-lang
        meta_language = self.soup.find(
            "meta",
            {
                "http-equiv": lambda x: x and x.lower() == "content-language",
                "content": True,
            },
        )
        if meta_language:
            language = meta_language.get("content").split(",", 1)[0]
            if language:
                candidate_languages[language] = True

        # If other langs exist, remove 'en' commonly generated by HTML editors
        if len(candidate_languages) > 1:
            candidate_languages.pop("en", None)

        # Return the first candidate language
        return candidate_languages.popitem(last=False)[0]

    def ingredients(self):
        raise NotImplementedError("This should be implemented.")

    def instructions(self) -> str:
        """instructions to prepare the recipe"""
        raise NotImplementedError("This should be implemented.")

    def instructions_list(self) -> List[str]:
        """instructions to prepare the recipe"""
        return [
            instruction
            for instruction in self.instructions().split("\n")
            if instruction
        ]

    def ratings(self):
        raise NotImplementedError("This should be implemented.")

    def author(self):
        raise NotImplementedError("This should be implemented.")

    def cuisine(self):
        raise NotImplementedError("This should be implemented.")

    def description(self):
        raise NotImplementedError("This should be implemented.")

    def reviews(self):
        raise NotImplementedError("This should be implemented.")

    def links(self):
        invalid_href = {"#", ""}
        links_html = self.soup.findAll("a", href=True)

        return [link.attrs for link in links_html if link["href"] not in invalid_href]

    def site_name(self):
        meta = self.soup.find("meta", property="og:site_name")
        return meta.get("content") if meta else None

    def to_json(self):
        json_dict = {}
        public_method_names = [
            method
            for method in dir(self)
            if callable(getattr(self, method))
            if not method.startswith("_") and method not in ["soup", "links", "to_json"]
        ]
        for method in public_method_names:
            try:
                json_dict[method] = getattr(self, method)()
            except Exception:
                pass
        return json_dict



from recipe_scrapers.dr import Dr
from tests import ScraperTest


class TestDrMeScraper(ScraperTest):

    scraper_class = Dr

    def test_host(self):
        self.assertEqual("dr.dk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Millionaires’ Shortbread", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Brdr. Price")

    def test_total_time(self):
        self.assertEqual(120, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://asset.dr.dk/ImageScaler03/?url=http%3A%2F%2Fmad-recipe-pictures-dr-dk.s3.amazonaws.com%2Fprod%2Frecipe%2Fmillionaires-shortbread-jpg-1597218581.jpg",
            self.harvester_class.image(),
        )

    def test_language(self):
        self.assertEqual("da-dk", self.harvester_class.language())

    def test_ingredients(self):
        self.assertEqual(
            [
                "130 g mel",
                "50 g farin",
                "2 tsk. maizena",
                "Et nip salt",
                "120 g smør",
                "1 spsk. koldt vand",
                "1 æggeblomme",
                "1 dåse sød kondenseret mælk",
                "100 g farin",
                "100 g smør",
                "2 spsk. lys sirup",
                "1 tsk. vanilje ekstrakt",
                "Et nip salt",
                "170 g mørk chokolade",
                "1 stor spsk. smør",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Opskriften giver cirka 16 styks, afhængig af størrelsen.

I en foodprocessor kommes mel, farin, maizena og salt. Kør til alt er blandet.

Kom smørret ved og kør til det ligner brødkrummer. Tilsæt koldt vand og æggeblomme og kør til dejen lige samler sig.

Kom dejen ud på et melet bord og saml den hurtigt. Kom køkkenfilm omkring den og lad den hvile i køleskabet i en times tid.

Rul den derpå ud på et melet bord og placer den i bunden af en firkantet bageform (24 x 24 cm.) der er foret med bagepapir. Prik dejen med en gaffel og bag den i en 180 grader varm ovn i ca. 20 minutter.

Til karamellen kommes den kondenserede mælk i en gryde sammen med farin, smør, sirup, vanille og salt. Bring massen i kog og lad den koge roligt til den når 110 grader på et sukkertermometer - eller til massen er tyk og gyldenbrun. Hæld karamellem over den stadig lune kagebund.

Lad derpå kagen køle helt af.

Smelt chokoladen i et vandbad sammen med smørret, bland godt og hæld så chokoladen ovenpå karamellen. Jævn ud med en spatel. Sæt kagen på køl.

Tag kagen ud af formen og fjern bagepapiret. Skær kagen i små firkanter som opbevares på køl til de skal serveres.""",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
import re

from bs4 import BeautifulSoup

from ._abstract import AbstractScraper
from ._utils import get_yields, normalize_string


class MyKitchen101(AbstractScraper):
    @classmethod
    def host(cls):
        return "mykitchen101.com"

    def author(self):
        return self.soup.find("a", {"rel": "author"}).get_text()

    def title(self):
        return self.soup.find("h1", {"class": "entry-title"}).get_text()

    def yields(self):
        return get_yields(self.soup.find("p", string=re.compile("分量：")).get_text())

    def image(self):
        return self.schema.image()

    def ingredients(self):
        soup = BeautifulSoup(str(self.soup), features="html.parser")
        ingredients = (
            soup.find(name="p", string=re.compile("材料：")).find_next("ul").find_all("li")
        )
        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        soup = BeautifulSoup(str(self.soup), features="html.parser")
        instructions = soup.find(name="p", string=re.compile("做法：")).find_all_next("p")
        return "\n".join(
            [
                normalize_string(instruction.get_text())
                for instruction in instructions
                if instruction.get_text()[:1].isdigit()
            ]
        )



from recipe_scrapers.realfoodtesco import RealFoodTesco
from tests import ScraperTest


class TestRealFoodTescoScraper(ScraperTest):

    scraper_class = RealFoodTesco

    def test_host(self):
        self.assertEqual("realfood.tesco.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Tesco Real Food", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Roasted cauliflower tagine", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://realfood.tesco.com/media/images/1400x919-Roasted-cauliflower-tagine-d35191d9-d524-482a-8db8-11b122f4cbf0-0-1400x919.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 cauliflower, trimmed and broken into florets",
                "1 tbsp olive oil",
                "1 tbsp ras el hanout (or harissa seasoning)",
                "1 carrot, sliced on the diagonal",
                "2 red onions, thickly sliced",
                "3 garlic cloves, finely sliced",
                "400g tin chopped tomatoes",
                "400g tin chickpeas, drained and rinsed",
                "80g reduced-salt pitted green olives, halved",
                "200g wholewheat couscous",
                "½ reduced-salt vegetable stock cube, made up to 250ml",
                "120ml fat-free yogurt",
                "15g fresh parsley, roughly chopped",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Preheat the oven to gas 7, 220°C, fan 200°C. Toss the cauliflower with ½ tbsp oil and ½ tbsp ras el hanout in a baking dish, then roast for 20-25 mins until tender and golden.
Meanwhile, heat ½ tbsp oil in a large, lidded saucepan over a medium heat. Add the onions, carrot and garlic and cook for 5 mins, then stir in ½ tbsp ras el hanout and cook for 1 min. Add the tomatoes, chickpeas, olives and 200ml boiling water and bring to the boil. Reduce the heat to low, cover and simmer for 15-20 mins until the veg is cooked through and the sauce has thickened. Remove from the heat and stir in the cauliflower.
Put the couscous in a heatproof bowl, pour over the stock, cover and set aside for 5 mins, then fluff up with a fork. Divide between 4 plates and top with the tagine, yogurt and parsley to serve.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.thespruceeats import TheSpruceEats
from tests import ScraperTest


class TestTheSpruceEatsScraper(ScraperTest):

    scraper_class = TheSpruceEats

    def test_host(self):
        self.assertEqual("thespruceeats.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.thespruceeats.com/doner-kebab-recipe-4171703",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Homemade Doner Kebab")

    def test_total_time(self):
        self.assertEqual(140, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.thespruceeats.com/thmb/cvoXe0xiaLRO4FO8T8OTXWSw5K0=/1500x0/filters:no_upscale():max_bytes(150000):strip_icc()/aqIMG_1498fsq-5b343910c9e77c001a218bd0.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "For the Kebab:",
                "1 pound ground lamb (or 1/2 pound each of ground lamb and ground beef)",
                "1 egg",
                "4 cloves garlic (peeled and finely minced)",
                "1 teaspoon ground cumin",
                "1 teaspoon ground coriander",
                "1 teaspoon smoked paprika",
                "1 teaspoon dried oregano",
                "1/2 teaspoon salt",
                "1/4 teaspoon ground black pepper",
                "1 tablespoon oil (for the skillet, plus extra for the pan)",
                "For the Sandwich:",
                "4 rounds of pita (or naan or flatbread)",
                "1 cup assorted lettuce",
                "1 large tomato (sliced)",
                "1/2 seedless English cucumber (sliced)",
                "1/4 large red onion (peeled and sliced)",
                "Tzatziki sauce or tahini sauce",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Gather the ingredients.\nPreheat the oven to 350 F.\nIn a large bowl, combine the ground lamb, egg, garlic, ground cumin, ground coriander, smoked paprika, dried oregano, salt, and black pepper.\nPlace the mixture into an oiled 9 x 5-inch loaf pan and cook in the oven for approximately 30 minutes or until the top is a light golden brown.\nYou can slice the loaf immediately if you like but, for best results, cool completely, wrap in aluminum foil, and refrigerate until firm.\nTo reheat, add a little olive oil to a large skillet, slice the loaf very thinly and crisp up the slices in the hot pan for a few minutes.\nAssemble the sandwiches with pita, warmed and toasted through. Spread on some tzatziki or tahini sauce, add lettuce, tomato, cucumber, onion, and top with more sauce.\nServe and enjoy.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.owenhan import OwenHan
from tests import ScraperTest


class TestOwenHanScraper(ScraperTest):

    scraper_class = OwenHan

    def author(self):
        self.assertEqual("Owen Han", self.harvester_class.host())

    def test_host(self):
        self.assertEqual("owen-han.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual("Chicken Bacon Ranch", self.harvester_class.title())

    def test_image(self):
        self.assertEqual(
            "http://static1.squarespace.com/static/627be79397093e2de753b260/627c408602fed77ca384eb11/63120c4090e9bf706973d712/1662127792157/IMG_2037.jpg?format=1500w",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "Baguette",
                "1 chicken breast",
                "3 slices bacon",
                "Salt + Pepper",
                "Oregano",
                "Pepper Jack cheese",
                "Avocado",
                "Pickled onions",
                "1 cup plain non fat greek yogurt",
                "1/3 cup buttermilk",
                "1 lemon juiced",
                "1 tbsp fresh parsley",
                "1 tbsp fresh dill",
                "2 tsp garlic powder",
                "2 tsp onion powder",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            [
                "Cut the chicken and bacon into even thin strips.",
                "Place bacon in a skillet over medium heat. Cook until it's almost completely cooked, but not too crispy.",
                "Remove the bacon and add the chicken to the skillet.",
                "Season with oregano and lightly with salt. Cook until no longer pink.",
                "Turn off the heat, add the bacon back to the skillet and combine.",
                "Add pepper jack on top and cover to melt.",
                "For the ranch combine all the ingredients to a bowl and mix well to combine.",
                "Assemble: to a toasted baguette add the chicken and bacon then top with sliced avocado pickled onions, and ranch. Add some mixed greens to the other half. ",
                "Close the sandwich, cut in half and enjoy!",
            ],
            self.harvester_class.instructions(),
        )



from recipe_scrapers.kochbar import Kochbar
from tests import ScraperTest


class TestKochbarScraper(ScraperTest):

    scraper_class = Kochbar

    def test_host(self):
        self.assertEqual("kochbar.de", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.kochbar.de/rezept/549118/Ligurisches-Huehnerragout-mit-Zucchini-Spezzatino-con-zucchine.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Ligurisches Hühnerragout mit Zucchini – Spezzatino con zucchine",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Antareja")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://ais.kochbar.de/kbrezept/549118_1087108/1200x1200/ligurisches-huehnerragout-mit-zucchini-spezzatino-con-zucchine-rezept-bild-nr-2.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "250 g Hühnerbrust, ohne Knochen",
                "3 mittelgross Knoblauchzehen, frisch",
                "2 TL Zitronensaft",
                "2 EL Olivenöl, extra vergine",
                "1 TL Oregano, frisch oder TK",
                "4 kleine Zwiebelchen, rot",
                "2 mittelgross Knoblauchzehen, frisch",
                "200 g Zucchini, grün",
                "1 mittelgross Kartoffel, festkochend",
                "40 g Karotte",
                "4 mit Tomaten, rot, vollreif",
                "2 EL Schnittsellerie-Stängel, frisch oder TK",
                "1 kleine Peperoni, rot, mittelscharf",
                "3 EL Olivenöl",
                "2 TL Rosmarin, frisch oder TK",
                "6 Oliven, grün, kernlos",
                "2 Prise Pfeffer, schwarz, frisch aus der Mühle",
                "2 EL Schnittsellerie-Blätter, frisch oder TK",
                "50 g Tomatensaft",
                "50 g Weißwein, trocken",
                "1 TL Zucker",
                "2 TL (gestrichen) Hühnerbrühe, Kraftbouillon",
                "Blüten und Blätter",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Die Hühnerbrust in ca. 2 x 3 cm große Stücke schneiden. Die Knoblauchzehen in eine Schale auspressen und die restlichen Zutaten zur Marinade zufügen. Die Hühnerstücke in der Marinade für ca. 1 Stunde marinieren.\nIn der Zwischenzeit für das Gemüse die Zwiebelchen und die Knoblauchzehen an beide Enden kappen, schälen und in kleine Stücke schneiden. Den Zucchino waschen, an beiden Enden kappen und längs halbieren. Die Hälften quer in ca. 8 mm dicke Scheiben schneiden. Die Kartoffel waschen, schälen, längs halbieren, die Hälften längs halbieren und quer vierteln. In Salzwasser in 15 Minuten gar kochen, das Wasser abgießen und die Kartoffeln bereit halten.\nDie Karotte waschen, an beiden Enden kappen und schälen. Mit einer groben Raspel die entsprechende Menge von unten her abraspeln. Bei den Tomaten die Stiele entfernen, häuten, vierteln und entkernen. Die Viertel längs und quer halbieren.\nDie frische Schnittsellerie waschen, trocken schütteln und die makellosen Blätter abzupfen, zerkleinern, 2 EL bereit halten und den Rest tieffrieren. Die makellosen Stiele quer in ca. 3 mm breite Röllchen schneiden und 2 EL davon bereithalten. Die restlichen Röllchen tieffrieren. TK-Ware abwiegen und auftauen lassen. Die frischen, roten Peperoni waschen, die Stiele entfernen, diagonal in ca. 6 mm breite Stücke schneiden und die Körner belassen.\nFür die Würze die Oliven längs vierteln und mit den restlichen Zutaten bereit halten. Die Zutaten für die Sauce mischen und rühren, bis der Zucker gelöst ist. Die Hühnerstücke abseihen und die Marinadenreste in die Sauce mischen.\nIn einer tiefen Pfanne oder Wok das Olivenöl erhitzen bis es duftet. Die Zwiebelchen und die Knoblauchzehen zugeben und 30 Sekunden braten, dann die Hühnerstücke zufügen und 3 Minuten scharf braten. Zucchini, Karotte, Schnittsellerie-Stängel und Peperoni dazugeben und 1 Minute pfannenrühren. Die Kartoffel- und Tomatenstücke zusammen mit der Würze untermischen. Noch 1 Minute pfannenrühren und dann mit der Sauce ablöschen. Mit Deckel 3 Minuten köcheln lassen.\nMit Salz und Pfeffer abschmecken, auf die angewärmten Servierteller verteilen, garnieren, sofort gut warm servieren und genießen.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.rutgerbakt import RutgerBakt
from tests import ScraperTest


class TestRutgerBaktScraper(ScraperTest):
    scraper_class = RutgerBakt
    test_file_name = "rutgerbakt_2"

    def test_host(self):
        self.assertEqual("rutgerbakt.nl", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Rutger van den Broek", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Abrikozenvlaai", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(120, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual(None, self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://rutgerbakt.nl/wp-content/uploads/2020/04/abrikozenvlaai-scaled-1920x1080-c-default.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 recept vlaaideeg",
                "1 kilo verse rijpe abrikozen of 750 gr abrikozenhelften, uitgelekt uit blik",
                "¾ recept banketbakkersroom",
                "suiker, optioneel, om te bestrooien",
                "boter, om in te vetten",
                "bloem, voor het werkblad",
                "150 gr abrikozenjam",
                "20 gr amandelschaafsel",
                "poedersuiker, om te bestuiven",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Maak eerst de banketbakkersroom en het vlaaideeg.\nAls je verse abrikozen gebruikt moeten deze eerst nog voorbereid worden, om de schil te verwijderen. Snijd de abrikozen kruislings in en leg ze 1 à 2 minuten in kokend water. Leg ze vervolgens in ijskoud water om het kookproces te stoppen. Verwijder de schil van de abrikozen, halveer ze en verwijder de pit.\nVet een vlaaivorm met een doorsnede van 28 centimeter en een hoogte van 3 centimeter in met boter. Kneed het deeg nog kort door en rol het dan op een bebloemd werkblad uit tot een ronde lap, die ruim over de vlaaivorm past. Bekleed de vorm met het deeg en zorg dat het overal goed aansluit. Verwijder het overhangende deeg door met een deegroller over de vorm te rollen of met behulp van een mes.\nVerwarm de oven voor op 220 °C. Klop de afgekoelde banketbakkersroom los met (hand)mixer met garde(s) en verdeel deze over de met deeg beklede vlaaivorm. Rangschik de abrikozenhelften met de bolle kanten omhoog op de banketbakkersroom, zodat de hele vlaai bedekt is. Als je wilt kun je nog wat suiker over de abrikozen strooien.\nAbrikozenvlaai bakken\nLaat de vlaai nog 15 minuten rusten en bak deze vervolgens in 25 tot 35 minuten goudbruin en gaar. Laat de vlaai na het bakken een halfuur afkoelen in de vorm en plaats hem daarna op een rooster om verder af te koelen.\nVerlaag de oventemperatuur naar 160 °C en spreid het amandelschaafsel uit over een met bakpapier beklede bakplaat. Rooster het schaafsel 6-8 minuten tot het goudbruin is.\nVerwarm de abrikozenjam en bestrijk daarmee de afgekoelde abrikozen vlaai. Strooi het amandelschaafsel over de vlaai en bestuif de rand licht met poedersuiker.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            "Met dit recept kun je zelf een heerlijke abrikozenvlaai bakken! Deze vlaai is het lekkerst met verse abrikozen, maar buiten het seizoen kun je die ook vervangen door abrikozen uit blik. Ik gebruik voor deze vlaai een vulling van banketbakkersroom. Dat in combinatie met de frisse abrikozen en vlaaibodem levert een heerlijke abrikozentaart op! Wil je een abrikozenvlaai met slagroom maken? Garneer de afgekoelde vlaai dan met opgeklopte slagroom. Van dit abrikozenvlaai recept kun je 10-12 punten snijden. ",
            self.harvester_class.description(),
        )



from recipe_scrapers.sunbasket import SunBasket
from tests import ScraperTest


class TestSunBasketScraper(ScraperTest):

    scraper_class = SunBasket

    def test_host(self):
        self.assertEqual("sunbasket.com", self.harvester_class.host())

    def test_host_domain(self):
        self.assertEqual("sunbasket.co.uk", self.harvester_class.host(domain="co.uk"))

    def test_title(self):
        self.assertEqual(
            "Lemongrass-turkey salad with rice noodles and pear",
            self.harvester_class.title(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Sun Basket")

    def test_total_time(self):
        self.assertEqual(15, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "5 ounces flat rice noodles",
                "Sun Basket lemongrass paste (lemongrass - extra virgin olive oil - garlic - ginger - turmeric)",
                "3 or 4 organic radishes (about ¼ pound total)",
                "1 organic Asian or other pear",
                "4 or 5 sprigs organic fresh mint",
                "Sun Basket Thai dressing base (maple syrup - lime juice - fish sauce)",
                "3 ounces organic baby kale or other leafy greens",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "1: Cook the rice noodles - Bring a medium [large] sauce pot of water to a boil. Add the rice noodles and cook until just tender, 6 to 8 minutes. Drain and rinse with cold water, then return to the pot and set aside. While the water is heating and the noodles are cooking, start preparing the rest of the meal.",
                    "2: Prep and cook your protein - Ground meat: Cut a small corner from the ground meat packaging and drain off any excess liquid. Transfer to a plate; pat dry with a paper towel. Shrimp: Rinse and drain the shrimp. Pat dry on a paper-towel-lined plate. Tofu: Pat the tofu dry with paper towels; cut into ½-inch cubes. Plant-based chick*n: Pat the chick*n dry with a paper towel. In a large frying pan over medium-high heat, warm 2 to 3 teaspoons oil until hot but not smoking. Add your protein and lemongrass paste, season lightly with salt and pepper, and cook, stirring occasionally and breaking up the ground meat if using, until the protein is lightly browned and cooked through, 2 to 4 minutes for regular shrimp; 4 to 6 minutes for jumbo shrimp, tofu, or plant-based chick*n; and 5 to 7 minutes for ground meat. Remove from the heat. Meanwhile, prepare the remaining ingredients.",
                    "3: Prep the remaining ingredients; assemble the salad - Cut the radishes in half, then cut the halves into thin half-moons. Peel the Asian pear, if desired; core and thinly slice. Strip the mint leaves from the stems; coarsely chop or tear the leaves. In a small bowl, stir together the Thai dressing base and 1 tablespoon [2 TBL] oil; season to taste with salt and pepper. In a large bowl, combine your protein, noodles, radishes, pear, mint, kale, and 2 to 3 tablespoons Thai dressing and toss to coat (set aside the remaining dressing for serving). Season to taste with salt and pepper.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "//cdn.sunbasket.com/100408c8-f908-404e-a494-099fad9fed44.jpg",
            self.harvester_class.image(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.pressureluckcooking import PressureLuckCooking
from tests import ScraperTest

# Data from https://pressureluckcooking.com/instant-pot-jeffreys-favorite-chicken/


class TestPressureLuckCookingScraper(ScraperTest):

    scraper_class = PressureLuckCooking

    def test_host(self):
        self.assertEqual("pressureluckcooking.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Jeffrey", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Instant Pot Jeffrey's Favorite Chicken", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Poultry", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://pressureluckcooking.com/wp-content/uploads/2023/01/Jeffreys-Favorite-Chicken-IG-1-scaled-720x720.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 1/2 - 2 pounds boneless, skinless chicken cutlets (breasts sliced into 1/4-inch thick fillets)",
                "1/4 cup of one of the following: all-purpose flour, whole wheat flour, coconut flour, or quinoa flour (with a few pinches garlic powder, peppers and salt mixed in with a fork)",
                "1/4 cup extra-virgin olive oil",
                "2 teaspoons ghee or 1 tablespoon salted butter, divided (see Jeff's Tips)",
                "2 large shallots, diced",
                "8 ounces baby bella or white mushrooms, sliced",
                "6 cloves (2 tablespoons) garlic, minced or pressed",
                "1/2 cup dry white wine (like a sauvignon blanc) or additional broth",
                "Juice of 1/2 lemon",
                "1/2 cup low-sodium chicken broth",
                "1 teaspoon Italian seasoning",
                "1 teaspoon seasoned salt",
                "5-8 ounces baby spinach",
                "1 tablespoon cornstarch + 1 tablespoon cold water",
                "1/4 cup any milk of your choice (or an unsweetened nondairy milk such as almond, oat or soy)",
                "1 (10-ounce) jar sun-dried tomatoes, drained and roughly chopped",
                "1 (14-ounce) can artichoke hearts, drained and chopped",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Dredge the chicken cutlets in the flour mixture so they're lightly coated and set aside on a plate.\nAdd the olive oil and half the butter/ghee, hit Sauté and Adjust to the More or High setting. After 3 minutes of heating, add the chicken to the pot in batches and sear on each side for about 45-60 seconds, until just ever so lightly browned. Use tongs to remove and rest the seared on a plate.\nAdd the remaining half of butter/ghee. (NOTE: If the olive oil is totally gone from searing the chicken, you can add 1 additional tablespoon). Once the butter's melted, add the shallots and mushrooms and sauté for 2 minutes. Add the garlic and sauté for 1 minute longer.\nAdd the wine (or additional 1/2 cup of broth) and lemon juice and simmer for 2 minutes, scraping the bottom of the pot to loosen any browned bits so it's nice and smooth.\nAdd the broth, Italian seasoning, seasoned salt and stir. Return the seared chicken to the pot and top with the spinach (if it seems piled high, don't worry. It cooks down to nothing).\nSecure the lid, movie the valve to the sealing position, and hit Cancel followed by Pressure Cook or Manual at High Pressure for 5 minutes. Quick release when done. Using tongs, transfer the chicken to a serving dish.\nMix together the cornstarch and water to form a slurry.\nHit Cancel followed by Sauté and Adjust to the More or High setting. Stir in the slurry and it will thicken the sauce immediately. Sit in the milk, sun-dried tomatoes and artichokes. Hit Cancel to turn the pot off.\nLadle the sauce over the chicken and serve!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.6, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("American", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "A fervent believer in giving folks what they ask for, I recently polled my readers for the next dish they wanted me to make. The winner was chicken and I felt it only fitting to finally put one of my favorite recipes that in my Lighter (blue) book on the blog. In fact, it's called Jeffrey's Favorite Chicken for a reason and seeing how it's tender cutlets in a light, yet rich and hearty lemon-wine sauce loaded with mushrooms, spinach, artichokes and sun-dried tomatoes, can you blame me? It can also easily be keto, paleo, gluten-free and dairy-free (see Jeff's Tips).",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class HeinzBrasil(AbstractScraper):
    @classmethod
    def host(cls):
        return "heinzbrasil.com.br"

    def title(self):
        return self.soup.find("h1", {"class": "krRDPrecName"}).get_text()

    def total_time(self):
        return 0

    def image(self):
        return self.soup.find("img", {"class": "krBanImg"})["src"]

    def ingredients(self):
        ingredients = self.soup.findAll("div", {"class": "krRDPIngreListText"})

        return [
            normalize_string(
                "{} {}".format(ingredient["qty"], ingredient["ingredientname"])
            )
            for ingredient in ingredients
        ]

    def instructions(self):
        instructions = (
            self.soup.find("div", {"class": "krRecipeMakeItText"})
            .findNext("div", {"class": "class"})
            .nextSibling
        )
        return normalize_string(instructions.get_text())



from recipe_scrapers.whatsgabycooking import WhatsGabyCooking
from tests import ScraperTest


class TestWhatsGabyCookingScraper(ScraperTest):

    scraper_class = WhatsGabyCooking

    def test_host(self):
        self.assertEqual("whatsgabycooking.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://whatsgabycooking.com/vegetarian-quinoa-bake/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Vegetarian Quinoa Bake")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Gaby Dalkin")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn.whatsgabycooking.com/wp-content/uploads/2017/10/WGC-Quinoa-Bake-copy-2.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 ½ cups uncooked multi-colored quinoa",
                "2 cups shredded colby jack cheese (divided)",
                "1 cup shredded mozzarella cheese (divided)",
                "1 cup canned black beans (rinsed and drained)",
                "1 cup frozen charred corn (trader joes)",
                "1 4.5-ounce can chopped green chiles",
                "1 ½ cup Gaby's chipotle or tomatillo salsa",
                "Kosher salt and pepper to taste",
                "Finely chopped scallions and cilantro as garnish",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 375 degrees F.\nCook the quinoa according to the package directions. Once cooked, remove from heat and transfer the cooked quinoa into a large bowl.\nFold in 1 1/2 cups of the shredded colby jack cheese, ½ cup of the shredded mozzarella, black beans, corn, green chiles and salsa. Season the entire mixture with salt and pepper and stir to combine.\nLightly spray a medium sized skillet with non-stick spray, and transfer the mixture into the skillet. Top with the remaining shredded cheeses and bake for about 20-25 minutes until the top layer of cheese is bubbly and melted.\nRemove the baking dish from the oven and garnish with green onions and cilantro and serve.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.archanaskitchen import ArchanasKitchen
from tests import ScraperTest


class TestArchanasKitchenScraper(ScraperTest):

    scraper_class = ArchanasKitchen

    def test_host(self):
        self.assertEqual("archanaskitchen.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Archana's Kitchen", self.harvester_class.author())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.archanaskitchen.com/classic-greek-salad-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Classic Greek salad Recipe")

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 items", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.archanaskitchen.com/images/archanaskitchen/0-Archanas-Kitchen-Recipes/2018/May-2/Classic_Greek_Salad_With_Olives_And_Feta_Cheese-7.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 tablespoons Extra Virgin Olive Oil",
                "4 tablespoons Lemon juice",
                "Salt and Pepper, to taste",
                "1 cup Iceberg lettuce, washed thoroughly and drained",
                "2 Tomatoes, cut into chunks",
                "1 English Cucumber, chopped",
                "1 Red onion, sliced",
                "1/2 cup Kalmatta olives",
                "100 grams Feta Cheese, crumbled",
                "2 sprig Parsley leaves, chopped (optional)",
                "1/2 cup Kalmatta olives",
                "100 grams Feta Cheese, crumbled",
                "2 sprig Parsley leaves, chopped",
                "Salt and Pepper, to taste",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "We begin making the Classic Greek salad Recipe by washing the lettuce thoroughly and draining off any excess water. Tear them into bite size pieces and keep it aside.\nCut all the vegetables and keep it ready, add all the salad ingredients(lettuce,cucumber, tomatoes,red onion) except feta & olives into a large mixing bowl.\nIn another bowl whisk in the dressing ingredients till it combines. Drizzle this on top of the salad mixture and give it a toss.Check for salt and pepper and add if required.\nServe the salad on a flat plate topped up with some crumbled feta cheese and olives.\nServe the Classic Greek salad Recipe as a Party Appetizer followed by a Vegan Moussaka to relish a Greek Meal.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.9, self.harvester_class.ratings())
